package com.tripgis.mobile;

import android.app.Activity;
import android.app.Application;
import android.app.Dialog;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.Toast;

import com.tripgis.mobile.data.LocationRecord;
import com.tripgis.mobile.data.TripRecord;
import com.tripgis.mobile.utils.TripUtilities;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Created by pk on 7/12/17.
 */

public class TripGisApplication extends Application implements SharedPreferences.OnSharedPreferenceChangeListener
{
    private SharedPreferences prefs;

    private TripFileManager fileManager;
    private int savedRecord;

    public static final String TAG = TripGisApplication.class.getSimpleName();

    // Request Codes to return when asking user for Permission to access system settings
    public static final int REQUEST_LOCATION_PERMISSION = 100;
    public static final int REQUEST_WRITE_DATA_PERMISSION = 110;

    /**
     * Overridden from Application.
     */
    @Override
    public void onCreate()
    {
        super.onCreate();
        Log.d( TAG, "onCreate()" );
        // TODO - when we get here
//        prefs = PreferenceManager.getDefaultSharedPreferences( this );
//        prefs.registerOnSharedPreferenceChangeListener( this );
    }

    public void loadFileManager()
    {
        try
        {
            fileManager = new TripFileManager();
        }
        catch( IOException e )
        {
            Log.e( TAG, "Failed to create file '" + TripFileManager.DEFAULT_TRIP_NAME + "', that's gonna be a problem..." );
            Log.e( TAG, "ERROR: " + e );
            System.exit(2);
        }
    }

    // TODO
    // https://developer.android.com/reference/android/preference/PreferenceActivity.html
    // https://stackoverflow.com/questions/6822319/what-to-use-instead-of-addpreferencesfromresource-in-a-preferenceactivity

    /**
     * Overridden from Application.
     */
    @Override
    public void onTerminate()
    {
        super.onTerminate();
        Log.d( TAG, "onTerminate()" );
    }

    /**
     * Overridden from OnSharedPreferenceChangeListener.
     *
     * @param sharedPreferences
     * @param s
     */
    @Override
    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String s)
    {
        Log.d( TAG, "onSharedPreferenceChanged()" );
        // Reset any data that may be affected by this data
    }

    /**************************************************************************
     *    Dealing with Permissions
     *************************************************************************/

    /**
     * Determine if permission has been granted for the application to access
     * Android settings. If not, a request is made, and if denied, well, TBD...
     *
     * References:
     *    https://developer.android.com/training/permissions/requesting.html
     *
     *    TODO - https://stackoverflow.com/questions/37526680/wait-for-user-permission (splash screen)
     *
     * @param permission the Manifest.permission String representing permission
     * @param requestCode app-specific return code (onRequestPermissionsResult can use)
     * @return true if the permission has been granted
     */
    public boolean isPermissable( Activity context, String permission, int requestCode )
    {
        Log.d( TAG, "isPermissiable( <Activity>, " + permission + ", " + requestCode + ")" );
        // See if permission has already been granted, request it if not
        if( ContextCompat.checkSelfPermission( this, permission ) != PackageManager.PERMISSION_GRANTED )
        {
            //ActivityCompat.requestPermissions(MapsActivity.this, new String[]{permission}, requestCode);
            ActivityCompat.requestPermissions( context, new String[]{permission}, requestCode );
            try {
                Thread.sleep( 5000 );
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            // Perform a second check after the request
            return ContextCompat.checkSelfPermission(this, permission) == PackageManager.PERMISSION_GRANTED;
        }

        return true;        // Permission granted if we get this far
    }

    /**************************************************************************
     *    Handling Trip Data
     *************************************************************************/

    public TripFileManager getFileManager()
    {
        return fileManager;
    }

    public TripRecord getTripRecord()         { return fileManager.getTrip(); }

    public void setTripRecord( TripRecord t ) throws IOException
    {
        Log.d( TAG, "setTripRecord( " + t + " )" );
        fileManager.setTrip( t );
    }

    public boolean isDefaultTrip() { return fileManager.getTrip().getName().equals( TripFileManager.DEFAULT_TRIP_NAME ); }

    public void showSelectTripDialog( Dialog dialog )
    {
        Log.i( TAG, "Opening Dialog" );
        dialog.show();
    }

    /**
     *
     * References:
     *    https://stackoverflow.com/questions/2634991/android-1-6-android-view-windowmanagerbadtokenexception-unable-to-add-window
     *
     * @param activity
     * @param item
     * @return
     */
    public Dialog setupTripDialog( Activity activity, MenuItem item )
    {
        Log.d( TAG, "setupTripDialog( <Activity>, " + item + " )" );

        try
        {
            final Dialog dialog = new Dialog( activity );
            dialog.setContentView( R.layout.select_trip_dialog );

            ArrayList<String> list = TripFileManager.getAvailableTrips();
            if( list.size() < 1 )
                Log.i( TAG, "No trips available" );

            else
            {
                Spinner dropDown = (Spinner) dialog.findViewById( R.id.action_select_trip );
                ArrayAdapter<String> tripAdapter = new ArrayAdapter<String>( this, android.R.layout.simple_spinner_item, list );
                tripAdapter.setDropDownViewResource( android.R.layout.simple_spinner_dropdown_item );
                dropDown.setAdapter( tripAdapter );

                Button okButton = (Button) dialog.findViewById( R.id.action_ok );
                okButton.setOnClickListener( (View.OnClickListener) activity );
                Button cancelButton = (Button) dialog.findViewById( R.id.action_cancel );
                cancelButton.setOnClickListener( (View.OnClickListener) activity );

                item.setEnabled( true );
                Log.d( TAG, item + " Enabled: " + item.isEnabled() );
            }

            return dialog;
        }
        catch( IOException e )
        {
            setError( TAG, "Failed to find available trips " + e );
            item.setEnabled( false );
            return null;
        }
    }

    public String getSelectedTripName( Dialog dialog )
    {
        Spinner dropDown = (Spinner) dialog.findViewById( R.id.action_select_trip );
        return (String) dropDown.getSelectedItem();
    }

    public void dismissDialog( Dialog dialog )
    {
        dialog.dismiss();
    }

    /**
     * References:
     *    https://stackoverflow.com/questions/16252269/how-to-sort-a-arraylist-in-java
     *
     * @param tripName
     * @param items
     * @return
     */
    public void moveItems( String tripName, ArrayList<Integer> items ) throws IOException
    {
        Log.d( TAG, "moveItems( " + tripName + ", " + items.size() + " )" );

        TripFileManager tfm = new TripFileManager( tripName );
        Log.d( TAG, "Copying " + items.size() + " items to trip - " + tfm.getTrip() );

        ArrayList<LocationRecord> move_to = tfm.getTrip().getSavedLocations();
        ArrayList<LocationRecord> move_from = getSavedLocations();

        for( int i = items.size() - 1; i >= 0; i-- )
        {
            Log.d( TAG, "Moving [" + items.get( i ) + "] = " + move_from.get( items.get( i ) ) );
            move_to.add( move_from.remove( items.get( i ).intValue() ) );
        }

        TripUtilities.sortLocationRecords( move_to );

        tfm.write( move_to, true );
        fileManager.updateSavedLocations( move_from );
    }

    /**************************************************************************
     *    Handling Saved Locations
     *************************************************************************/

    public void setRecordForActivity( int position ) { this.savedRecord = position; }

    public int getRecordPositionForActivity() { return savedRecord; }
    public LocationRecord getRecordForActivity() { return getSavedLocations().get( savedRecord ); }

    public ArrayList<LocationRecord> getSavedLocations(){ return fileManager.getSavedLocations(); }

    /**************************************************************************
     *    Text & Error Handling
     *************************************************************************/


    public void setError( String tag, String text )
    {
        Log.e( tag, text );
        Toast.makeText( this, text, Toast.LENGTH_SHORT).show();
    }

    public void notifyUser( String tag, String text )
    {
        Log.i( tag, text );
        Toast.makeText( this, text, Toast.LENGTH_SHORT).show();
    }
}
