package com.tripgis.mobile.activity;

// NOTE: If R is highlighted red below, it is due to the XML files in the res/
//    directory not compiling. The following Stack Overflow discusses:
//    https://stackoverflow.com/questions/885009/r-cannot-be-resolved-android-error
import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.IntentSender;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.VectorDrawable;
import android.os.Build;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.os.Bundle;

import com.tripgis.mobile.TripGisApplication;
import com.tripgis.mobile.R;
import com.tripgis.mobile.data.LocationRecord;
import com.tripgis.mobile.data.TripRecord;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

// Added     compile 'com.google.android.gms:play-services-location:11.0.2'
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;

import android.location.Location;
import android.support.v4.content.ContextCompat;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.Toast;

import java.io.IOException;
import java.util.ArrayList;

// TODO
// DONE Allow user to select different position on map
// Add single-click to popup basic location details
// Add double-click feature to open Save Location Form
// Create save location button to open Save Location Form
//   Open file with file manager, FTP it to home network
// DONE* Add View overlay to menu - read file and add to map - have to toggle it off/on to update data set
// DONE Save Location Form
// DONE  Name: Text Field
// DONE  Date: <auto-entered>
// DONE  Lat/Lon: <auto-entered>
//   Photo Button - open phone app, take picture
//   Trash Button
//      Ponder how to delete file
//   Return to Map
// Popup
//   Name, Date, Lat/Lon, Photo, Trash
//      Photo opens Gallery, adds to record
//      Ponder how to append record
//   X Close Button
// DONE* Setup app to work without being tethered - not sure how to release apk or install it on another device
// FUTURE:
//   Attempt to georeference
// DONE  Create Trip Form
// DONE*     Create Select Trip Form - form looks like shit
// DONE  Create Manage Locations Form - select, delete, move
//   Build in navigation or link to maps to provide them
//   Network connectivity
//      Need to be able to auto-sync, specify settings, manual upload
//   Add Weather fields: temp, conditions
// NICE TO DO:
//   Ponder SQLite instead of CSV
//   Tie in to weather service
//   Tie in to Contacts

// The following link:
//http://blog.teamtreehouse.com/beginners-guide-location-android
// Provides all steps necessary to build a basic application

// Basic reference for adding an App bar and its features:
//     https://developer.android.com/training/appbar/setting-up.html

// FragmentActivity is set by default for MapActivity from Studio
//     Use AppCompatActivity to add an ActionBar
//     https://developer.android.com/training/appbar/index.html

// Releasing project
// https://developer.android.com/studio/publish/preparing.html
// https://developer.android.com/studio/publish/app-signing.html

/**
 * MapsActivity is the main 'driver' class for the Android Application. It was
 * developed using Android Studio vice Java Eclipse due to the every changing
 * nature of the Android SDK world.
 *
 * NOTES: Initial setup was using Exclipse (1), but only three months later,
 *    the documentation changed course and these steps no longer worked. After
 *    additional searching, I found how to use my phone as a test bench (2).
 *    This uses USB to tether the device and allows debugging on the intended
 *    platform - pretty slick!
 *
 *    To use the Google API, a key is required (3). This link shows existing
 *    keys and allows creation of a new one if desired. Using the (free) API
 *    grants several thousand accesses daily, which seems adequate.
 *
 *    Additional map options and features available are further discussed (4).
 *
 * References:
 *    1. https://askubuntu.com/questions/318246/complete-installation-guide-for-android-sdk-adt-bundle-on-ubuntu
 *    2. https://developer.android.com/training/basics/firstapp/running-app.html
 *    3. https://console.developers.google.com/apis/credentials?project=pro-bruin-172600
 *    4. http://www.androidhive.info/2013/08/android-working-with-google-maps-v2/
 */
public class MapsActivity extends AppCompatActivity implements
        OnMapReadyCallback, GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener, LocationListener,
        GoogleMap.OnMapClickListener, View.OnClickListener {

    private GoogleMap mMap;
    private FusedLocationProviderClient client;

    private GoogleApiClient googleClient;
    private LocationRequest locRequest;
    private Dialog dialog;

    private static final long MIN_ACCURACY = 300; // Number of seconds since last update

    // Indicates onCreate() finished
    // This flag will be used to distinguish between startup and user activities
    private boolean initializationComplete = false;
    private TripGisApplication application;

    private Toolbar toolbar;
    private Menu menu;

    private boolean monitorPosition = true;     // Show the current position blue ball
    private LocationRecord lastMarker = null;   // Store the last selected marker
    private boolean viewOverlays = false;       // Show stored positions

    private static final int LOCATION_REQUEST_CODE = 101;
    private static final int SELECT_REQUEST_CODE = 102;
    private static final int CREATE_TRIP_REQUEST_CODE = 103;
    private static final int TRIP_REQUEST_CODE = 104;

    // Simplified name to use in log messages and data messaging
    public static final String TAG = MapsActivity.class.getSimpleName();
    private static final int CONNECTION_FAILURE_RESOLUTION_REQUEST = 9000;

//    private String markerName = "Marker in Sydney";
//    private double lat = -34;
//    private double lon = 151;

    /**
     * Initialization function for when app is first created. Used to set up
     * API calls and basic save_location. Actual interactions and data presentation
     * will be covered elsewhere. Refer to onResume and onPause.
     *
     * Overridden from AppCompatActivity.
     *
     * @param savedInstanceState not used
     */
    @Override
    protected void onCreate( Bundle savedInstanceState )
    {
        super.onCreate(savedInstanceState);     // Call parent for default behavior
        Log.d( TAG, "onCreate( <Bundle> )" );
        setContentView(R.layout.app_layout);    // Access the associated layouts XML

        application = (TripGisApplication) getApplication();

        // Obtain the SupportMapFragment and get notified when the map is ready to be used.
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);

        // Attempt to initialize the Google API client
        googleClient = new GoogleApiClient.Builder( this )
                .addConnectionCallbacks( this )
                .addOnConnectionFailedListener( this )
                .addApi( LocationServices.API ).build();

        // Ensure the user has allowed the application to use location services
        if( !application.isPermissable( this, Manifest.permission.WRITE_EXTERNAL_STORAGE, TripGisApplication.REQUEST_WRITE_DATA_PERMISSION ) )
            Log.e( TAG, "Permission to write data files denied! Aww, shucks!!" );

        // Ensure the user has allowed the application to use location services
        if( !application.isPermissable( this, Manifest.permission.ACCESS_FINE_LOCATION, TripGisApplication.REQUEST_LOCATION_PERMISSION ) )
            Log.e( TAG, "Permission to access location services denied! Tough Tootles!!" );

        application.loadFileManager();

        /**********************************************************************
         * Prepare the Toolbar
         *********************************************************************/
        // Toolbar is the main title on top of the activity
        toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        toolbar.showOverflowMenu();
        toolbar.setTitle( R.string.title_activity_maps );
//        getSupportActionBar().setDisplayHomeAsUpEnabled( true );

        // Enable the Up button
/*        ActionBar ab = getSupportActionBar();
        ab.setDisplayHomeAsUpEnabled(true); */

        // Create the LocationRequest object
        locRequest = LocationRequest.create()
                .setPriority( LocationRequest.PRIORITY_HIGH_ACCURACY )
                .setInterval(10 * 1000)        // 10 seconds, in milliseconds
                .setFastestInterval(1 * 1000); // 1 second, in milliseconds

        initializationComplete = true;      // All future logic is based on user-interaction
    }

    /**
     * Called when the application is made active (after onCreate on startup).
     *
     * Overridden from Activity.
     *
     */
    @Override
    protected void onResume()
    {
        super.onResume();
        Log.d( TAG, "onResume()" );

        if( menu != null )
            dialog = application.setupTripDialog( this, menu.findItem( R.id.action_open_trip ) );

        // Reinitialize connections to services
        googleClient.connect();
        if( mMap != null )
            repaint();
    }

    /**
     * Called when the application is "minimized"
     *
     * Overridden from Activity.
     *
     */
    @Override
    protected void onPause()
    {
        super.onPause();
        Log.d( TAG, "onPause()" );

        // Close all service connections
        // Save battery life by closing all connections onPause()
        if (googleClient.isConnected())
        {
            LocationServices.FusedLocationApi.removeLocationUpdates(googleClient, this);
            googleClient.disconnect();
        }
    }

    /**
     * Called when the application is destroyed. There are no guarantees the
     * application will be destroyed on closure. This would be considered a
     * "clean" shutdown.
     *
     * Overridden from Activity.
     */
    @Override
    protected void onDestroy()
    {
        super.onDestroy();
        Log.i( TAG, "Closing program...bye!" );
    }

    /**
     * References:
     *    https://stackoverflow.com/questions/43142062/how-to-change-icon-colors-in-android-studio
     *    https://stackoverflow.com/questions/32437603/how-to-set-tint-in-navigationview-to-some-icons
     *    https://stackoverflow.com/questions/32924986/change-fill-color-on-vector-asset-in-android-studio (old)
     *
     * @param color
     * @param itemId
     */
    private void setColor( int color, int itemId )
    {
        Log.d( TAG, "setPositionColor( " + color + ", " + itemId + " )" );
        String sColor = Integer.toString( color );

        if( color == Color.GREEN )      sColor = "GREEN";
        else if( color == Color.GRAY )  sColor = "GRAY";
        else if( color == Color.BLACK ) sColor = "BLACK";
        else if( color == Color.WHITE ) sColor = "WHITE";
        else if( color == Color.BLUE )  sColor = "BLUE";
        else if( color == Color.RED )   sColor = "RED";

        MenuItem item = menu.findItem( itemId );
        Log.i( TAG, "Setting " + item + " color to " + sColor );
        item.getIcon().setColorFilter( color, PorterDuff.Mode.SRC_IN );
    }

    @TargetApi(21)
    @Deprecated
    private void setIcon( int itemId, int iconId )
    {
        VectorDrawable vdIcon = (VectorDrawable) getDrawable( iconId );
        //MenuView.ItemView item = (MenuView.ItemView) findViewById( itemId );
        //item.setIcon( vdIcon );

        if( iconId == R.drawable.ic_my_location_green_24dp )
        {
            Log.i( TAG, "Changing icon of " + itemId + " to holo_green_dark" );
            vdIcon.setColorFilter(ContextCompat.getColor( getApplicationContext(),
                    android.R.color.holo_green_dark), PorterDuff.Mode.MULTIPLY);
        }
        else
        {
            Log.i( TAG, "Changing color of " + iconId + " to black" );
            vdIcon.setColorFilter(ContextCompat.getColor( getApplicationContext(),
                    android.R.color.black), PorterDuff.Mode.MULTIPLY);
        }
    }

    /**
     *
     * Overridden from OnMapClickListener.
     *
     * References:
     *    https://developers.google.com/android/reference/com/google/android/gms/maps/GoogleMap.OnMapClickListener
     *
     * @param latLng
     */
    @Override
    public void onMapClick( LatLng latLng )
    {
        Log.d( TAG, "onMapClick( " + latLng + " )" );
        LocationRecord record = new LocationRecord(
                LocationRecord.getCurrentTime(),
                latLng,
                "",
                null );
        Log.i( TAG, "Received click at " + record );

        lastMarker = record;
        repaint();
        addMarker( record );
        setColor( Color.BLACK, R.id.action_cpos );
    }

    /**
     * Manipulates the map once available.
     * This callback is triggered when the map is ready to be used.
     * This is where we can add markers or lines, add listeners or move the camera. In this case,
     * we just add a marker near Sydney, Australia.
     * If Google Play services is not installed on the device, the user will be prompted to install
     * it inside the SupportMapFragment. This method will only be triggered once the user has
     * installed Google Play services and returned to the app.
     */
    @Override
    public void onMapReady( GoogleMap googleMap )
    {
        Log.d( TAG, "onMapReady( <map> )" );
        mMap = googleMap;

        // TODO - Change map type
        //mMap.setMapType(GoogleMap.MAP_TYPE_NORMAL);
        //mMap.setMapType(GoogleMap.MAP_TYPE_HYBRID);
        //mMap.setMapType(GoogleMap.MAP_TYPE_SATELLITE);
        //mMap.setMapType(GoogleMap.MAP_TYPE_TERRAIN);
        //mMap.setMapType(GoogleMap.MAP_TYPE_NONE);

        // TODO - Set Zoom Controls
        //mMap.getUiSettings().setZoomControlsEnabled(false); // true to enable

        // TODO - Disable Zoom Gestures
        //mMap.getUiSettings().setZoomGesturesEnabled(false);

        // TODO - Disable Rotate Gestures
        //mMap.getUiSettings().setRotateGesturesEnabled(true);

        // TODO - Set Compass
        mMap.getUiSettings().setCompassEnabled(true);

/*        client = LocationServices.getFusedLocationProviderClient(this);
        try {
            client.getLastLocation()
                    .addOnSuccessListener(this, new OnSuccessListener<Location>() {
                        @Override
                        public void onSuccess(Location location) {
                            // Got last known location. In some rare situations this can be null.
                            if (location != null) {
                                LatLng cpos = new LatLng( location.getLatitude(), location.getLongitude() );
                                mMap.moveCamera(CameraUpdateFactory.newLatLng(cpos));
                                mMap.addMarker(new MarkerOptions().position(cpos).title("Last Position"));
                            }
                        }
                    });
        } catch( SecurityException e ) {
            System.out.println( "BAD" );
        }
*/
        // http://wptrafficanalyzer.in/blog/adding-marker-on-touched-location-of-google-maps-using-android-api-v2-with-supportmapfragment/
//        // Add a marker in Sydney and move the camera
//        LatLng sydney = new LatLng(-34, 151);
//        mMap.moveCamera(CameraUpdateFactory.newLatLng(sydney));
//        mMap.addMarker(new MarkerOptions().position(sydney).title("Marker in Sydney"));

        // Define the handler for single-tapping on the Map
        mMap.setOnMapClickListener( this );
    }

    /**
     * This function is called once the location service completes initialization.
     * If the current position cannot be acquired, then a regular request to
     * Google location services (via the API) will be made. The
     * onLocationChanged() function will handle all returns from the API.
     *
     * References:
     *    https://developers.google.com/android/reference/com/google/android/gms/location/LocationRequest
     *    https://developer.android.com/reference/android/location/LocationManager.html
     *
     * @param bundle the received bundle data (not used)
     */
    @Override
    public void onConnected( @Nullable Bundle bundle )
    {
        Log.d( TAG, "onCollected( <Bundle> )" );
        getCurrentPosition( true );
    }

    @Override
    public void onConnectionSuspended( int i )
    {
        Log.d( TAG, "onConnectionSuspended( " + i + " )" );
    }

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult)
    {
        Log.d( TAG, "onConnectionFailed( <ConnectionResult> )" );
        if (connectionResult.hasResolution()) {
            try {
                // Start an Activity that tries to resolve the error
                connectionResult.startResolutionForResult(this, CONNECTION_FAILURE_RESOLUTION_REQUEST);
            } catch (IntentSender.SendIntentException e) {
                e.printStackTrace();
            }
        } else {
            Log.i(TAG, "Location services connection failed with code " + connectionResult.getErrorCode());
        }
    }

    private LatLng getCurrentPosition( boolean repoCamera )
    {
        Log.d( TAG, "getCurrentPosition( " + repoCamera + " )" );

        lastMarker = null;              // Clear any user-action markers
        Location location = null;

        try
        {
            // Attempt to get the last known location from the Google API
            location = LocationServices.FusedLocationApi.getLastLocation( googleClient );

            if( location == null )                     // Location can be null if first call or errors
            {
                setColor( Color.RED, R.id.action_cpos );

                // User requested position, and we don't know it
                if( initializationComplete )
                {
                    Log.e( TAG, "Could not obtain last position" );
                }

                // Send a request to receive updates to location
                else
                {
                    Log.w(TAG, "Last Location: null, requesting updates from API...");
                    LocationServices.FusedLocationApi.requestLocationUpdates(googleClient, locRequest, this);
                }
            }
            else
            {
                long diff = Math.abs( location.getTime() - System.currentTimeMillis() ) / 1000;
                if( diff > MIN_ACCURACY )
                {
                    application.setError( TAG, "Google client appears to be offline! Last update " + diff + " secs ago" );
                }

                changeLocation( location, repoCamera );            // update the Fragment with current position
                setColor( Color.GREEN, R.id.action_cpos );

                // Enable current location marker (if desired, on by default)
                if( monitorPosition )
                    mMap.setMyLocationEnabled( true ); // false to disable

                return new LatLng( location.getLatitude(), location.getLongitude() );
            }
        }
        catch( SecurityException e )
        {
            application.setError( TAG, "No Permissions to find location from smart phone!" );
        }

        // Position could not be found
        return null;
    }

    // Called by requestLocationUpdate() (googleApi)

    /**
     *
     * References:
     *    https://developer.android.com/reference/android/location/LocationListener.html
     *
     * Overridden from LocationListener.
     *
     * @param location
     */
    @Override
    public void onLocationChanged( Location location )
    {
        Log.d( TAG, "onLocationChanged( " + location + " )" );
        changeLocation( location, true );
        handlePositionUpdates();
    }

    public void setMonitorPosition( boolean flag ) { monitorPosition = flag; }

    /**
     * Determines if updates should continue to come in from Location services.
     * In general, it is desirable to leave this off to save battery power.
     */
    public void handlePositionUpdates()
    {
        Log.d( TAG, "handlePositionUpdates()" );
        try
        {
            // If monitoring enabled, ensure updates have been scheduled
            if (monitorPosition)
                LocationServices.FusedLocationApi.requestLocationUpdates(googleClient, locRequest, this);
            else
                LocationServices.FusedLocationApi.removeLocationUpdates(googleClient, this);
        }
        catch( SecurityException e )
        {
            Log.e( TAG, "Permission services disabled - " + e.toString() );
        }
    }

    private void repaint()
    {
        Log.d( TAG, "repaint()" );
        // https://www.chupamobile.com/tutorial-android/integrating-google-maps-in-android-app-53
        mMap.clear();

        //if( lastMarker == null )
        //    getCurrentPosition( false );

        if( viewOverlays )
            for (LocationRecord record : application.getSavedLocations() )
                addMarker( record );
    }

    /**
     * References:
     *    https://stackoverflow.com/questions/42365658/custom-marker-in-google-maps-in-android-with-vector-asset-icon
     *    https://stackoverflow.com/questions/33696488/getting-bitmap-from-vector-drawable
     *    https://www.flaticon.com/free-icon/push-pin_211749#term=push%20pin&page=2&position=86
     *
     * @param location
     * @param repoCamera
     */
    private void changeLocation( Location location, boolean repoCamera )
    {
        Log.d( TAG, "changeLocation( " + location + ", " + repoCamera + " )" );

        // Add Marker
        double currentLatitude = location.getLatitude();
        double currentLongitude = location.getLongitude();
        LatLng latLng = new LatLng( currentLatitude, currentLongitude );

        MarkerOptions options = new MarkerOptions()
                .position( latLng )         // Location of marker
                .infoWindowAnchor( 0, 0 )   //??
                .title("I am here!");       // Text to associate

        // Changing marker icon
        //
        //
        //options.icon( BitmapDescriptorFactory.fromResource( R.drawable.ic_my_location_black_24dp ) );
      options.icon( BitmapDescriptorFactory.fromBitmap( getBitmap( this, R.drawable.ic_push_pin )));
//        options.icon( BitmapDescriptorFactory.fromBitmap( getBitmap( this, R.drawable.ic_bookmark_black_24dp )));

        //repaint();
        mMap.addMarker( options );

        if( repoCamera )
        {
            // Update Camera Position
            //        mMap.moveCamera( CameraUpdateFactory.newLatLng( latLng ) );

            // Option 2 -
            CameraPosition cameraPosition = new CameraPosition.Builder()
                    .target(latLng)           // Position to center on
                    .zoom(12)                 // Zoom/Extent (1 - 21)
                    // bearing, tilt, target
                    .build();

            mMap.animateCamera(CameraUpdateFactory.newCameraPosition(cameraPosition));
        }
    }

    // https://stackoverflow.com/questions/42365658/custom-marker-in-google-maps-in-android-with-vector-asset-icon
    // https://stackoverflow.com/questions/33696488/getting-bitmap-from-vector-drawable
    public Bitmap getBitmap( Context context, int drawableId )
    {
        Log.d( TAG, "getBitmap( <Context>, " + drawableId + " )" );
        Drawable drawable = ContextCompat.getDrawable(context,  drawableId);
        if (drawable instanceof BitmapDrawable) {
            return BitmapFactory.decodeResource(context.getResources(), drawableId);
        } else if (drawable instanceof VectorDrawable) {
            // https://stackoverflow.com/questions/33696488/getting-bitmap-from-vector-drawable
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
                drawable = (DrawableCompat.wrap(drawable)).mutate();
            }

            Bitmap bitmap = Bitmap.createBitmap(drawable.getIntrinsicWidth(),
                    drawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888);
            Canvas canvas = new Canvas(bitmap);
            drawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
            drawable.draw(canvas);

            return bitmap;
        } else {
            throw new IllegalArgumentException("unsupported drawable type");
        }
    }

    /**
     * Adds a marker to the current Google Map Fragment. Does not update camera
     * (extent or position) of map.
     *
     * @param record the data to display
     * @return true if all is well
     */
    public void addMarker( LocationRecord record )
    {
        Log.d( TAG, "addMarker( " + record + " )" );

        // create marker
        MarkerOptions marker = new MarkerOptions()
                .position( record.getLocation() )
                // alpha, anchor, draggable
                // flat, icon, infoWindowAnchor
                // rotation, snippet, visible, zIndex
                .title( record.getName() );

        // Change marker color to distinguish from current location
        marker.icon( BitmapDescriptorFactory.defaultMarker( BitmapDescriptorFactory.HUE_GREEN ) );

        // fromAsset(String assetName) – Loading from assets folder
        // fromBitmap (Bitmap image) – Loading bitmap image
        // fromFile (String path) – Loading from file
        // fromResource (int resourceId) – Loading from drawable resource

        // adding marker
        mMap.addMarker( marker );
    }

    /**
     * Setup the menu. This is called once during starting of the application.
     *
     * References:
     *    https://stackoverflow.com/questions/7066657/android-how-to-dynamically-change-menu-item-text-outside-of-onoptionsitemssele
     *
     * @param menu the menu and its map_items from the /res/menu/*.xml file
     * @return always true
     */
    @Override
    public boolean onCreateOptionsMenu( Menu menu )
    {
        // Define the menu and its map_items
        Log.d( TAG, "onCreateOptionsMenu( " + menu + " )" );
        getMenuInflater().inflate(R.menu.map_items, menu);

        // It is recommended to save an internal copy of the menu for editing if needed
        this.menu = menu;
        dialog = application.setupTripDialog( this, menu.findItem( R.id.action_open_trip ) );

        return true;
    }

    /**
     * Handler for children activities to respond to requests from main. It is
     * necessary to use startActivityForResult() to call the child, and to
     * call setResult( requestCode ) and finish() in the child. It is
     * recommended to use RESULT_OK or RESULT_CANCELED to indicate outcome.
     *
     * References:
     *    https://stackoverflow.com/questions/2906925/how-do-i-pass-an-object-from-one-activity-to-another-on-android
     *    https://stackoverflow.com/questions/920306/sending-data-back-to-the-main-activity-in-android
     *
     * Overridden from Activity.
     *
     * @param requestCode the code of the child program.
     * @param resultCode the Activity response code
     * @param data intent data (bundle) containing response data
     */
    @Override
    public void onActivityResult( int requestCode, int resultCode, Intent data )
    {
        super.onActivityResult(requestCode, resultCode, data);  // Default to parent behavior
        Log.d( TAG, "onActivityResult( " + requestCode + ", " + resultCode + ", <Intent> )" );

        // Handle request by the code received
        switch( requestCode )
        {
            // SaveLocationActivity responses
            case( LOCATION_REQUEST_CODE ):
                {
                    // Success indicates user clicked Save button
                    if( resultCode == Activity.RESULT_OK )
                    {
                        Log.i( TAG, "Received Save Location Data" );

                        // Store the data received
                        savePosition( (LocationRecord) data.getParcelableExtra( LocationRecord.TAG ) );

                        // TODO - the following code should be part of a handler
                        try
                        {
                            application.getFileManager().refreshSavedLocations();
                            repaint();
                        }
                        catch( IOException e )
                        {
                            application.setError( TAG, "Failed to repaint - " + e );
                        }
                    }

                    // Failure indicates Cancel/Back action - do nothing
                    else if( resultCode == Activity.RESULT_CANCELED )
                        Log.i( TAG, "Received Save Location Cancel" );

                    break;
                }
            // SaveLocationActivity responses
            case( TRIP_REQUEST_CODE ):
            {
                // Success indicates user clicked Save button
                if( resultCode == Activity.RESULT_OK )
                {
                    Log.i( TAG, "Received Trip Data" );

                    // Store the data received
                    // NOTE: This is done prior to returning for now
                    //application.setTripRecord( (TripRecord) data.getParcelableExtra( LocationRecord.TAG ) );
                    updateTrip( application.getTripRecord() );

                    // TODO - the following code should be part of a handler
                    try
                    {
                        application.getFileManager().refreshSavedLocations();
                        repaint();
                    }
                    catch( IOException e )
                    {
                        application.setError( TAG, "Failed to repaint - " + e );
                    }
                }

                // Failure indicates Cancel/Back action - do nothing
                else if( resultCode == Activity.RESULT_CANCELED )
                    Log.i( TAG, "Received Trip Cancel" );

                break;
            }
        }
    }

    /**
     * Overridden from OnClickListener.
     *
     * References:
     *    https://www.mkyong.com/android/android-spinner-drop-down-list-example/
     *
     * @param view
     */
    @Override
    public void onClick( View view )
    {
        Log.d( TAG, "onClick( " + view + " )" );

        // Determine which button was pressed
        switch( view.getId() )
        {
            case R.id.action_ok:
                Log.d( TAG, "action_ok" );
                TripRecord tripRecord = new TripRecord( application.getSelectedTripName( dialog ) );
                if( updateTrip( tripRecord ) )
                    application.dismissDialog(dialog);
                return;                                 // Always exit a case
            case R.id.action_cancel:
                Log.i( TAG, "action_cancel" );
                application.dismissDialog( dialog );
                return;                                 // Always exit a case

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                Log.e( TAG, "Unrecognized action - " + view.getId() );

        }
    }

    /**
     * References:
     *    https://stackoverflow.com/questions/3438276/how-to-change-the-text-on-the-action-bar
     *
     * @param tripRecord
     * @return
     */
    private boolean updateTrip( TripRecord tripRecord )
    {
        Log.d( TAG, "updateTrip( " + tripRecord + " )" );
        try
        {
            Log.d( TAG, "Setting trip - " + tripRecord );
            application.setTripRecord( tripRecord );
            tripRecord = application.getTripRecord();

            Log.d( TAG, "Old Title: " + getSupportActionBar().getTitle() + ", New: " + tripRecord );
            getSupportActionBar().setTitle( tripRecord.getName() );
            updateTripMenuItems();
            return true;
        }
        catch ( IOException e )
        {
            application.setError( TAG, "Failed to assign trip to " + tripRecord + " - " + e );
            return false;
        }
    }

    // AppCompatActivity
    @Override
    public boolean onOptionsItemSelected( MenuItem item )
    {
        Log.d( TAG, "onOptionsItemSelected( " + item + " )" );
        Intent intent;

        switch( item.getItemId() )
        {
//            case R.id.action_settings:
            //               // User chose the "Settings" item, show the app settings UI...
            //             return true;

            case R.id.action_cpos:
                Log.d( TAG, "action_cpos" );
                repaint();

                // Current position is desired location
                if( lastMarker == null )
                {
                    Log.i(TAG, "Save Action Requested (CPOS)");
                    sendPositionToSaveActivity();
                }

                // Another location selected, revert back to current
                else
                {
                    LatLng cpos = getCurrentPosition( true );
                }

                return true;

            case R.id.action_save:
                Log.d( TAG, "action_save" );
                sendPositionToSaveActivity();
                return true;

            case R.id.action_set_end_time:
                Log.d( TAG, "action_mark_end_time" );
                setColor( Color.GRAY, item.getItemId() );
                setEndTime();

                // Flash the icon for 0.1s to show "haptic" response
                // NOTE: If this is too long, you get gripes in the monitor
                //   "I/Choreographer: Skipped 36 frames!  The application may be doing too much work on its main thread."
                try
                {
                    Thread.sleep( 100 );
                }
                catch( InterruptedException e )
                {
                    Thread.currentThread().interrupt();
                }

                setColor( Color.BLACK, item.getItemId() );
                return true;

            case R.id.action_view_overlay:
                Log.d( TAG, "action_view_overlay" );
                viewOverlays = !viewOverlays;
                repaint();
                return true;

            case R.id.action_view_saved:
                Log.d( TAG, "action_view_saved" );
                intent = new Intent( this, SelectLocationActivity.class );
                intent.putParcelableArrayListExtra( LocationRecord.TAG, application.getSavedLocations() );
                Log.i( TAG, "Sending " + application.getSavedLocations().size() +
                        " map_items to " + SelectLocationActivity.TAG );
                startActivityForResult( intent, SELECT_REQUEST_CODE );
                return true;

            case R.id.action_create_trip:
                Log.d( TAG, "action_create_trip" );
                intent = new Intent( this, TripActivity.class );

                Log.i( TAG, "Calling " + TripActivity.TAG + " = " + item.getTitle() );
//                if( item.getTitle() == getString( R.string.title_create_trip ) )
//                    startActivityForResult(intent, CREATE_TRIP_REQUEST_CODE);

//                else
//                {
                Log.d( TAG, "Trip: " + application.getTripRecord() + ", Default: " + application.isDefaultTrip() );
                if( !application.isDefaultTrip() )
                {
                    Log.d( TAG, "Passing " + application.getTripRecord() );
                    intent.putExtra( TripRecord.TAG, application.getTripRecord() );
                }


                startActivityForResult( intent, TRIP_REQUEST_CODE );
//                }
                return true;

            case R.id.action_open_trip:
                Log.d( TAG, "action_open_trip" );
                application.showSelectTripDialog( dialog );
                return true;

            case R.id.action_close_trip:
                Log.d( TAG, "action_close_trip" );
                updateTrip( null );
                return true;

            case R.id.action_exit:
                Log.d( TAG, "action_exit" );
                finish();                                           // Close the Activity
                return true;                                        // Always exit a case

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);

        }
    }

    /**
     * Saves the user-desired position to the data store. If user has selected
     * a position on the map, lastMarker is set and will be used. If user has
     * just opened the program, or has selected the CPOS menu option, the
     * curren position will be used - if it can be found.
     */
    private void savePosition( LocationRecord record )
    {
        Log.d( TAG, "savePosition( " + record + " )" );

        // Save a record if there is a valid position selected
        if( record != null )
        {
            try
            {
                Log.i( TAG, "Saving position - " + record );
                application.getFileManager().addSavedLocation( record );
            }
            catch( IOException e )
            {
                application.setError( TAG, "Failed to save " + record + "!" );
            }
        }
    }

    private void updateTripMenuItems()
    {
        Log.d( TAG, "updateTripMenuItems()" );
        MenuItem item = menu.findItem( R.id.action_create_trip );
        MenuItem item2 = menu.findItem( R.id.action_close_trip );

        // Save a record if there is a valid position selected
        if( !application.isDefaultTrip() )
        {
            item.setTitle( R.string.title_view_trip );
            item2.setEnabled( true );
        }
        else
        {
            item.setTitle(R.string.title_create_trip);
            item2.setEnabled( false );
        }
    }

    private void setEndTime()
    {
        Log.d( TAG, "setEndTime()" );
        ArrayList<LocationRecord> saved = application.getSavedLocations();
        if( saved == null || saved.size() == 0 )
        {
            application.setError(TAG, "No saved locations!");
            return;
        }

//        LocationRecord record = saved.get( saved.size() - 1 );
        LocationRecord record = saved.get( 0 );

        try
        {
            record.setEndTime( LocationRecord.getCurrentTime() );
//            application.getFileManager().read();
            application.notifyUser( TAG, "Set end time: " + record.toDisplayString() );
//            application.getFileManager().read();
            application.getFileManager().updateSavedLocations( saved );
//            application.getFileManager().read();
        }
        catch( IOException e )
        {
            application.setError( TAG, "Failed to set end time for " + record + "!" );
        }

    }

    //private void savePosition() { savePosition( lastMarker ); }

    private void sendPositionToSaveActivity()
    {
        Log.d( TAG, "sendPositionToSaveActivity()" );
        LocationRecord record = null;       // Data to store

        // User has selected a point on the map
        if( lastMarker != null )
            record = lastMarker;            // Use the user-specified location

        else
        {
            // Request the last good position from location services
            LatLng cpos = getCurrentPosition( false );

            if( cpos != null )              // If location can be found
                record = new LocationRecord( LocationRecord.getCurrentTime(), cpos, "", null );
        }

//        Intent intent = new Intent();
//        Bundle bundle = new Bundle();
//        bundle.putParcelable( LocationRecord.TAG, record );
//        intent.putExtras( bundle );
//        intent.setClass( this, SaveLocationActivity.class );

        Intent intent = new Intent( this, SaveLocationActivity.class );
        intent.putExtra( LocationRecord.TAG, record );
        Log.i( TAG, "Sending " + record + " to " + SaveLocationActivity.TAG );
        startActivityForResult( intent, LOCATION_REQUEST_CODE );
    }

/*    // Should probably attempt to receive a response...
    // https://stackoverflow.com/questions/31502650/android-m-request-permission-non-activity
    @Override
    public void onRequestPermissionsResult( int requestCode, String permissions[], int[] grantResults )
    {
        Log.d( TAG, "onRequestPermissionsResult()" );
        super.onRequestPermissionsResult( requestCode, permissions, grantResults );

        String permission = "";
        switch( requestCode )
        {
            case TripGisApplication.REQUEST_LOCATION_PERMISSION:
                permission = "Location";
                break;

            case TripGisApplication.REQUEST_WRITE_DATA_PERMISSION:
                permission = "File I/O";
                break;

                // other 'case' lines to check for other
            // permissions this app might request
        }

        if( grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED )
            Toast.makeText( this, permission + " Permission Granted", Toast.LENGTH_SHORT ).show();
        else
            Toast.makeText( this, permission + " Permission Denied", Toast.LENGTH_SHORT).show();
    } */

}
