package com.tripgis.mobile.activity;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.tripgis.mobile.FileManager;
import com.tripgis.mobile.TripGisApplication;
import com.tripgis.mobile.R;
import com.tripgis.mobile.data.LocationRecord;

// Added     compile 'com.google.android.gms:play-services-location:11.0.2'

/**
 * This class handles all interactions with the Save Location Form. When
 * the user selects Save Location from MapActivity, data is passed to this
 * class to be processed. Once complete, processing is returned to the main
 * program.
 *
 * TODO Weather
 *    https://code.tutsplus.com/tutorials/create-a-weather-app-on-android--cms-21587
 *    https://stackoverflow.com/questions/15425283/how-to-get-weather-at-current-location
 *    https://www.programmableweb.com/news/top-10-weather-apis/analysis/2014/11/13
 *
 * // Weather service...
 * https://darksky.net/app
 *    1000 requests/day, 0.0001 per forecast thereafter
 * https://darksky.net/dev/docs/time-machine
 * https://github.com/pequots34/Forecast
 * https://stackoverflow.com/questions/20275771/noaa-weather-api-on-android
 */
public class SaveLocationActivity extends AppCompatActivity implements View.OnClickListener,
        TextWatcher, TextView.OnEditorActionListener
{
    private EditText locationName;      // Name textfield
    private TextView date;              // Date textfield (read-only)
    private TextView latLong;           // Position textfield (read-only)
    private TextView endTime;           // End time textfield (read-only)
    private Button saveButton;          // Save button
    private Button endTimeButton;       // End Time button
    private Button cancelButton;        // Cancel button

    // Copy of data to be received/sent
    private LocationRecord record;

    private Toolbar toolbar;
    private Menu menu;
    private TripGisApplication application;

    // Simplified name to use in log messages and data messaging
    public static final String TAG = SaveLocationActivity.class.getSimpleName();

    /**
     * Initialization function for when app is first created. Used to set up
     * API calls and basic save_location. Actual interactions and data presentation
     * will be covered elsewhere. Refer to onResume and onPause.
     *
     * Overridden from Activity.
     *
     * @param savedInstanceState data passed from parent activity.
     */
    @Override
    public void onCreate( Bundle savedInstanceState )
    {

        super.onCreate(savedInstanceState);         // Call parent for default behavior
        setContentView(R.layout.save_location);     // Access the associated layouts XML

        application = (TripGisApplication) getApplication();

        // Read the passed LocationRecord object
        record = this.getIntent().getParcelableExtra( LocationRecord.TAG );

        if( record == null )
            record = application.getRecordForActivity();

        Log.i( TAG, "Received " + record + " from " + MapsActivity.TAG );

        // Initialize private variables for various "dynamic" content
        date = (TextView) findViewById( R.id.location_date );
        latLong = (TextView) findViewById( R.id.location_position );
        locationName = (EditText) findViewById( R.id.location_name );
        https://stackoverflow.com/questions/1489852/android-handle-enter-in-an-edittext
        locationName.setOnEditorActionListener( this );
        locationName.addTextChangedListener( this );
        endTime = (TextView) findViewById( R.id.location_end_time );
        saveButton = (Button) findViewById( R.id.save_button );
//FIXME        endTimeButton = (Button) findViewById( R.id.end_time_button );
        cancelButton = (Button) findViewById( R.id.cancel_button );

        // Define handlers for user-interaction
        saveButton.setOnClickListener( this );
        cancelButton.setOnClickListener( this );
//FIXME        endTimeButton.setOnClickListener( this );

        //https://stackoverflow.com/questions/8991522/how-can-i-set-the-focus-and-display-the-keyboard-on-my-edittext-programmatical
        locationName.requestFocus();
        locationName.setSelectAllOnFocus( true );           // Highlight field at outset
        InputMethodManager imm = (InputMethodManager) getSystemService( Context.INPUT_METHOD_SERVICE );
        imm.showSoftInput( locationName, InputMethodManager.SHOW_IMPLICIT );

        /**********************************************************************
         * Prepare the Toolbar
         ******************************************************************** */
        // Toolbar is the main title on top of the activity
        toolbar = (Toolbar) findViewById( R.id.save_toolbar );
        setSupportActionBar(toolbar);
        toolbar.showOverflowMenu();
        getSupportActionBar().setDisplayHomeAsUpEnabled( true );
    }

    /**
     * Called when the application is made active (after onCreate on startup).
     * Online documentation suggests that onCreate is not the appropriate
     * place to modify the default values for text. It's possible this is
     * no longer necessary, but better safe than sorry.
     *
     * References:
     *    https://stackoverflow.com/questions/4590957/how-to-set-text-in-an-edittext
     *
     * Overridden from Activity.
     */
    @Override
    protected void onResume()
    {
        super.onResume();                   // Call parent for default behavior

        // Set the textfields in the form to the values received by the parent
        date.setText( LocationRecord.getDateString( record.getDate() ) );
        latLong.setText( record.getLocationString() );
        locationName.setText( FileManager.readString( record.getName() ) );
        endTime.setText( LocationRecord.getDateString( record.getEndTime() ) );
    }


    /**
     * Setup the menu. This is called once during starting of the application.
     *
     * Overridden from Activity.
     *
     * @param menu the menu and its map_items from the /res/menu/*.xml file
     * @return always true
     */
    @Override
    public boolean onCreateOptionsMenu( Menu menu )
    {
        // Define the menu and its map_items
        Log.d( TAG, "onCreateOptionsMenu( <Menu> )" );
        getMenuInflater().inflate( R.menu.save_location_items, menu );

        // It is recommended to save an internal copy of the menu for editing if needed
        this.menu = menu;
        return true;
    }

    /**
     * Handler for user interaction with form.
     *
     * Overridden from View.OnClickListener
     *
     * @param view
     */
    @Override
    public void onClick( View view )
    {
        Log.d( TAG, "onClick( " + view + " )" );

        // Determine which button was pressed
        switch( view.getId() )
        {
            case R.id.save_button:                      // Save Button
                Log.d( TAG, "save_button" );
                if( saveLocation() )                    // Collect saved data
                    finish();                               // Close the Activity
                return;                                 // Always exit a case

            case R.id.cancel_button:                    // Cancel Button
                Log.i( TAG, "cancel_button" );
                setResult( Activity.RESULT_CANCELED );  // Indicate cancel
                finish();                               // Close the Activity
                return;                                 // Always exit a case

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                Log.e( TAG, "Unrecognized action - " + view.getId() );

        }
    }

    public boolean saveLocation()
    {
        // Create a new intent object to pass back to the parent
        Intent resultIntent = new Intent();

        // Ensure a name is specified
        if( locationName.getText().toString().isEmpty() )
        {
            application.setError( TAG, "Must specify a name to save!" );
            return false;
        }

        // Update data from form fields
        record.setName( locationName.getText().toString() );

        // Add the updated record data
        resultIntent.putExtra( LocationRecord.TAG, record );
        setResult( Activity.RESULT_OK, resultIntent );      // Indicate success
        return true;
    }

    /**
     * Handle user-selected action from the menu items (/res/menu/<file>.xml).
     *
     * Overridden from Activity.
     *
     * References:
     *    https://developer.android.com/training/implementing-navigation/ancestral.html
     *    https://developer.android.com/training/implementing-navigation/nav-drawer.html
     *    https://stackoverflow.com/questions/21805382/how-to-resume-activity-instead-of-restart-when-going-up-from-action-bar
     *
     * @param item the selected item
     * @return true if the action is processed
     */
    @Override
    public boolean onOptionsItemSelected( MenuItem item )
    {
        Log.d( TAG, "onOptionsItemSelected( " + item + " )" );
        switch (item.getItemId())
        {
            // Respond to the action bar's Up/Home button
            case android.R.id.home:
//                NavUtils.navigateUpFromSameTask( this );
                returnFromStack();
                return true;

            case R.id.action_save:
                Log.d( TAG, "action_save" );
                if( saveLocation() )                    // Collect saved data
                    finish();                           // Close the Activity
                return true;                            // Always exit a case

            case R.id.action_cancel:
                Log.d( TAG, "action_cancel" );
                setResult( Activity.RESULT_CANCELED );  // Indicate cancel
                finish();                               // Close the Activity
                return true;                            // Always exit a case

            default:
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                Log.e( TAG, "Unrecognized action - " + item.getItemId() );
                return super.onOptionsItemSelected( item );
        }
    }

    @TargetApi(16)
    private void returnFromStack()
    {
        navigateUpTo( getIntent() );
    }

    /**
     * Overridden from TextWatcher.
     *
     * References:
     *    https://developer.android.com/reference/android/text/TextWatcher.html
     *
     * @param s
     * @param start
     * @param before
     * @param count
     */
    @Override
    public void onTextChanged( CharSequence s, int start, int before, int count )
    {
        // Determine if enter was pressed

    }

    /**
     * Overridden from TextWatcher.
     *
     * @param s
     * @param start
     * @param count
     * @param after
     */
    @Override
    public void beforeTextChanged( CharSequence s, int start, int count, int after )
    {
    }

    /**
     * Overridden from TextWatcher.
     *
     * @param s
     */
    @Override
    public void afterTextChanged( Editable s )
    {
    }

    /**
     * Overridden from OnEditorActionListener.
     *
     * @param textView
     * @param i
     * @param keyEvent
     * @return
     */
    @Override
    public boolean onEditorAction( TextView textView, int i, KeyEvent keyEvent )
    {
        Log.d( TAG, "onEditorAction( " + textView.getImeActionLabel() + ", " + i + ", " + keyEvent.getKeyCode() );

        if( keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER )
        {
            saveLocation();                         // Collect saved data
        }

        return false;
    }
}
