package com.tripgis.mobile.activity;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

import com.tripgis.mobile.TripGisApplication;
import com.tripgis.mobile.R;
import com.tripgis.mobile.adapter.LocationAdapter;
import com.tripgis.mobile.data.LocationRecord;

import java.io.IOException;
import java.util.ArrayList;

/**
 * References:
 *    http://www.vogella.com/tutorials/AndroidListView/article.html
 */
public class SelectLocationActivity extends AppCompatActivity
        implements AdapterView.OnItemClickListener, AdapterView.OnItemLongClickListener, View.OnClickListener
{
    private TripGisApplication application;

    private Toolbar toolbar;
    private Menu menu;
    private Dialog dialog;

    private ListView listView;
    private LocationAdapter locationAdapter;
    private boolean selectAllToggle = true;

    public static final String TAG = SelectLocationActivity.class.getSimpleName();

    private static final int EDIT_REQUEST_CODE = 201;

    /**
     * Overridden from Activity.
     *
     * @param savedInstanceState
     */
    @Override
    protected void onCreate( Bundle savedInstanceState )
    {
        Log.d( TAG, "onCreate( <Bundle> )" );
        super.onCreate( savedInstanceState );
        /*setContentView( R.layout.activity_select_location );*/
        setContentView( R.layout.select_location );

        application = (TripGisApplication) getApplication();

        listView = (ListView) findViewById( R.id.list_location );
        listView.setOnItemClickListener( this );
        listView.setOnItemLongClickListener( this );
        listView.setChoiceMode( ListView.CHOICE_MODE_MULTIPLE );

        // TODO - add service to send updates
        // Read the passed LocationRecord object
        //ArrayList<LocationRecord> list = this.getIntent().getParcelableArrayListExtra( LocationRecord.TAG );
        ArrayList<LocationRecord> list = application.getSavedLocations();
        Log.i( TAG, "Received " + list.size() + " items from " + MapsActivity.TAG );

        resetAdapter( list );

        /**********************************************************************
         * Prepare the Toolbar
         ******************************************************************** */
        // Toolbar is the main title on top of the activity
        Toolbar toolbar = (Toolbar) findViewById( R.id.select_toolbar );
        setSupportActionBar(toolbar);
        toolbar.showOverflowMenu();
        getSupportActionBar().setDisplayHomeAsUpEnabled( true );

/*        FloatingActionButton fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Snackbar.make(view, "Replace with your own action", Snackbar.LENGTH_LONG)
                        .setAction("Action", null).show();
            }
        });*/
    }

    /**
     * Overridden from Activity.
     */
    @Override
    protected void onResume()
    {
        Log.d( TAG, "onResume()" );
        super.onResume();

        if( menu != null )
            dialog = application.setupTripDialog( this, menu.findItem( R.id.action_move_location ) );
    }

    /**
     * Overridden from Activity.
     */
    @Override
    public void onDestroy()
    {
        Log.d( TAG, "onDestroy()" );
        super.onDestroy();
    }

    /**
     * Setup the menu. This is called once during starting of the application.
     *
     * Overridden from Activity.
     *
     * @param menu the menu and its map_items from the /res/menu/*.xml file
     * @return always true
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Define the menu and its map_items
        Log.d( TAG, "onCreateOptionsMenu()" );
        getMenuInflater().inflate( R.menu.select_location_items, menu );

        // It is recommended to save an internal copy of the menu for editing if needed
        this.menu = menu;
        dialog = application.setupTripDialog( this, menu.findItem( R.id.action_move_location ) );
        return true;
    }

    /**
     * References:
     *    https://developer.android.com/guide/topics/ui/declaring-layout.html#FillingTheLayout
     *    https://developer.android.com/reference/android/widget/ListView.html
     *
     * @param list
     */
    private void resetAdapter( ArrayList<LocationRecord> list )
    {
        Log.d( TAG, "resetAdapter( " + list.size() + " )" );
        locationAdapter = new LocationAdapter( this, list );
        listView.setAdapter( locationAdapter );
    }

    /**
     * Handle user-selected action from the menu items (/res/menu/<file>.xml).
     *
     * Overridden from Activity.
     *
     * References:
     *    https://developer.android.com/training/implementing-navigation/ancestral.html
     *    https://developer.android.com/training/implementing-navigation/nav-drawer.html
     *    https://stackoverflow.com/questions/21805382/how-to-resume-activity-instead-of-restart-when-going-up-from-action-bar
     *
     * @param item the selected item
     * @return true if the action is processed
     */
    @Override
    public boolean onOptionsItemSelected( MenuItem item )
    {
        Log.d( TAG, "onOptionsItemSelected( " + item + " )" );

        switch (item.getItemId())
        {
            // Respond to the action bar's Up/Home button
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask( this );
                return true;

            case R.id.action_select_all_locations:
                Log.d( TAG, "action_select_all_locations" );
                selectAllToggle = !selectAllToggle;
                locationAdapter.selectAll( !selectAllToggle );
                return true;

            case R.id.action_delete_location:
                Log.d( TAG, "action_delete_locations" );
                deleteLocation();
                return true;

            case R.id.action_edit_location:
                Log.d( TAG, "action_edit_location" );

                if( locationAdapter.getSelectedItemCount() != 1 )
                    Toast.makeText( this, "Must select one and only one location to edit", Toast.LENGTH_SHORT).show();
                else
//                    sendRecordToSaveLocation( locationAdapter.getSelectedRecord() );
                    sendRecordToSaveLocation( locationAdapter.getSelectedItems().get( 0 ) );
                return true;

            case R.id.action_move_location:
                Log.d( TAG, "action_move_location" );

                if( locationAdapter.getSelectedItemCount() < 1 )
                    Toast.makeText( this, "Must select at least one location to move", Toast.LENGTH_SHORT).show();
                else
                    application.showSelectTripDialog( dialog );
                return true;

            default:
                Log.d( TAG, "default (unknown)" );
                // If we got here, the user's action was not recognized.
                // Invoke the superclass to handle it.
                return super.onOptionsItemSelected(item);
        }
    }

    /**
     * Overridden from OnClickListener.
     *
     * References:
     *    https://www.mkyong.com/android/android-spinner-drop-down-list-example/
     *
     * @param view
     */
    @Override
    public void onClick( View view )
    {
        Log.d( TAG, "onClick( <View> )" );
        ArrayList<Integer> items = locationAdapter.getSelectedItems();
        String tripName = application.getSelectedTripName( dialog );
        application.dismissDialog( dialog );

        try
        {
            application.moveItems( tripName, items );
            resetAdapter( application.getSavedLocations() );
        }
        catch( IOException e )
        {
            application.setError( TAG, "Failed to write to '" + tripName + "': " + e );
        }

        application.notifyUser( TAG, "Moved " + items.size() + " items to " + tripName );
    }


/*    public ArrayList<Integer> getSelectedItems()
    {
        Log.d( TAG, "getSelectedItems()" );

        SparseBooleanArray items = listView.getCheckedItemPositions();
        if( items.size() == 0 )
            return null;

        ArrayList<Integer> selectedList = new ArrayList<>();
        for( int pos = 0; pos < items.size(); pos++ )
        {
            if( items.get( pos ) )
            {
                Log.d( TAG, "Selected Record [" + pos + "]" );
                selectedList.add( pos );
            }
        }

        Log.d( TAG, "Number of selected items: " + selectedList.size() );
        return selectedList;
    } */

/*    public int getSelectedItemCount()
    {
        Log.d( TAG, "getSelectedItemCount()" );
        ArrayList<Integer> list = getSelectedItems();
        if( list == null )
            return -1;

        Log.d( TAG, "Number of selected items: " + list.size() );
        return list.size();
    } */

    @Deprecated
    private void sendRecordToSaveLocation( LocationRecord record )
    {
        Log.d( TAG, "sendRecordToSavedLocation( " + record + " )" );
        Intent intent = new Intent( this, SaveLocationActivity.class );
        intent.putExtra( LocationRecord.TAG, record );
        Log.i( TAG, "Sending " + record + " to " + SaveLocationActivity.TAG );
        startActivityForResult( intent, EDIT_REQUEST_CODE );
    }

    private void sendRecordToSaveLocation( int position )
    {
        Log.d( TAG, "sendRecordToSavedLocation( " + position + " )" );
        Intent intent = new Intent( this, SaveLocationActivity.class );
        LocationRecord record = locationAdapter.getItem( position );
        Log.i( TAG, "Sending " + record + " to " + SaveLocationActivity.TAG );
        application.setRecordForActivity( position );
        startActivityForResult( intent, EDIT_REQUEST_CODE );
    }


    /**
     * Handler for children activities to respond to requests from main. It is
     * necessary to use startActivityForResult() to call the child, and to
     * call setResult( requestCode ) and finish() in the child. It is
     * recommended to use RESULT_OK or RESULT_CANCELED to indicate outcome.
     *
     * References:
     *    https://stackoverflow.com/questions/2906925/how-do-i-pass-an-object-from-one-activity-to-another-on-android
     *    https://stackoverflow.com/questions/920306/sending-data-back-to-the-main-activity-in-android
     *
     * Overridden from Activity.
     *
     * @param requestCode the code of the child program.
     * @param resultCode the Activity response code
     * @param data intent data (bundle) containing response data
     */
    @Override
    public void onActivityResult( int requestCode, int resultCode, Intent data )
    {
        Log.d( TAG, "onActivityResult( " + requestCode + " , " + resultCode + ", <Intent> )" );
        super.onActivityResult(requestCode, resultCode, data);  // Default to parent behavior

        // Handle request by the code received
        switch( requestCode )
        {
            // SaveLocationActivity responses
            case( EDIT_REQUEST_CODE ):
            {
                Log.d( TAG, "edit_request_code" );

                // Success indicates user clicked Save button
                if( resultCode == Activity.RESULT_OK )
                {
                    Log.d( TAG, "result_ok" );
                    LocationRecord record = (LocationRecord) data.getParcelableExtra( LocationRecord.TAG );

                    try
                    {
                        if( locationAdapter.getSelectedItemCount() < 1 )
                            application.setError(TAG, "No locations to replace!");

                        // This will remove the
                        ArrayList<LocationRecord> list = locationAdapter.replaceRecord( record );
                        resetAdapter( application.getFileManager().updateSavedLocations( list ) );
                    }
                    catch( IOException e )
                    {
                        application.setError( TAG, "Failed to edit " + record );
                        locationAdapter.resetList( application.getSavedLocations() );
                    }
                }

                // Failure indicates Cancel/Back action - do nothing
                else if( resultCode == Activity.RESULT_CANCELED )
                    Log.d( TAG, "result_canceled" );

                break;
            }
        }
    }

    private void deleteLocation()
    {
        Log.d( TAG, "deleteLocation()" );

        try
        {
            if( locationAdapter.getSelectedItemCount() < 1 )
                application.setError(TAG, "No locations to delete!");

            ArrayList<LocationRecord> list = locationAdapter.deleteItems();
            resetAdapter( application.getFileManager().updateSavedLocations( list ) );
        }
        catch( IOException e )
        {
            application.setError( TAG, "Failed to delete items!" );
            locationAdapter.resetList( application.getSavedLocations() );
        }
    }

    /**
     * Overridden from OnItemClickListener.
     *
     *  References:
     *     https://stackoverflow.com/questions/5551042/onitemclicklistener-not-working-in-listview
     *
     * @param adapterView
     * @param view
     * @param pos
     * @param id
     */
    @Override
    public void onItemClick( AdapterView<?> adapterView, View view, int pos, long id )
    {
        Log.d( TAG, "onItemClick( " + pos + ", " + id + " )" );
        //final LocationRecord item = (LocationRecord) adapterView.getItemAtPosition( pos );
        //Log.i( TAG, "Selected item " + item );
        locationAdapter.setChecked( pos );
    }

    @Override
    public boolean onItemLongClick(AdapterView<?> adapterView, View view, int pos, long id )
    {
        Log.d( TAG, "onLongItemClick( <AdapterView>, <view>, " + pos + ", " + id + " )" );
        //final LocationRecord item = (LocationRecord) adapterView.getItemAtPosition( pos );
        //Log.i( TAG, "Selected item " + item );
        //sendRecordToSaveLocation( item );
////        sendRecordToSaveLocation( locationAdapter.getSelectedRecord() );
        sendRecordToSaveLocation( pos );
        return false;
    }
}
