package com.tripgis.mobile.adapter;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.BoolRes;
import android.support.annotation.LayoutRes;
import android.support.annotation.NonNull;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.util.SparseBooleanArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ListAdapter;
import android.widget.TextView;
import android.widget.Toast;

import com.tripgis.mobile.R;
import com.tripgis.mobile.activity.SelectLocationActivity;
import com.tripgis.mobile.data.LocationRecord;
import com.tripgis.mobile.utils.TripUtilities;

import org.w3c.dom.Text;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;


/**
 * Created by pk on 7/12/17.
 */

public class LocationAdapter extends ArrayAdapter<LocationRecord>
{
    private ArrayList<LocationRecord> list;
    private final Activity context;

    private int checkedCount = 0;

    private static final String TAG = LocationAdapter.class.getSimpleName();

    // http://www.vogella.com/tutorials/AndroidListView/article.html
    static class ViewHolder
    {
        public CheckBox cb;
        public TextView text;
    }
    private HashMap<Integer, View> checkboxHash;

    public LocationAdapter( Activity context, List<LocationRecord> objects )
    {
        super( context, R.layout.location_row, objects );
        Log.d( TAG, "constructor( " + objects.size() + " objects )" );

        this.context = context;
        resetList( (ArrayList<LocationRecord>) objects );
    }

    public void resetList( ArrayList<LocationRecord> list )
    {
        Log.d( TAG, "resetList( " + list.size() + " )" );
        this.list = list;
        checkboxHash = null;
        buildRows();
    }

    private void buildRows()
    {
        Log.d( TAG, "buildRows()" );
        checkboxHash = new HashMap<>();

        TripUtilities.sortLocationRecords( list );

        for( int pos = 0; pos < list.size(); pos++ )
        {
            Log.d( TAG, "Building checkbox item for position " + pos );
            LayoutInflater inflater = context.getLayoutInflater();
            View rowView = inflater.inflate( R.layout.location_row, null );

            // configure view holder
            ViewHolder viewHolder = new ViewHolder();
            viewHolder.text = (TextView) rowView.findViewById( R.id.list_location_text );
            viewHolder.cb = (CheckBox) rowView.findViewById( R.id.list_location_cb );
            rowView.setTag( viewHolder );

            viewHolder.cb.setOnCheckedChangeListener(
                    new CompoundButton.OnCheckedChangeListener()
                    {
                        @Override
                        public void onCheckedChanged( CompoundButton buttonView, boolean isChecked )
                        {
                            checkedCount += ( isChecked ? 1 : -1 );

                            //Model element = (Model) viewHolder.cb.getTag();
                            //element.setSelected( buttonView.isChecked() );
                        }
                    }
            );

            viewHolder.cb.setTag( list.get( pos ) );
            checkboxHash.put( pos , rowView );
            Log.d( TAG, "[" + pos + "]: " + checkboxHash.get( pos ).getTag() );
            Log.d( TAG, "[" + pos + "]: " + getViewHolder( checkboxHash.get( pos ) ) );
        }
    }

    private ViewHolder getViewHolder( View view )
    {
        Log.d( TAG, "getViewHolder( <View> )" );
        return ((ViewHolder) view.getTag() );
    }

    /**
     * Overridden from ArrayAdapter.
     *
     * References:
     *    https://stackoverflow.com/questions/17856363/checkbox-in-listview-multiple-select
     *
     * @param position
     * @param convertView
     * @param parent
     * @return
     */
    @Override
    public View getView( int position, View convertView, ViewGroup parent )
    {
        Log.d( TAG, "getView( " + position + ", <View>, <ViewGroup> )" );

        if( checkboxHash == null )
            buildRows();

        //View rowView = convertView;
        View rowView = checkboxHash.get( position );
        ViewHolder vh = getViewHolder( rowView );

//        // reuse views
//        //if( rowView == null )
//        if( checkboxHash.get( position ) == null )
//        {
//        }
//        else
//        {
////            vh = ((ViewHolder) rowView.getTag());
//        }

        Log.d( TAG, "Found existing object at position " + position );

        // fill data
//        ViewHolder holder = (ViewHolder) rowView.getTag();
//        ViewHolder holder = vh;

        //LocationRecord record = getItem( position );
        LocationRecord record = list.get( position );
        vh.text.setText( record.toDisplayString() );

        if( vh.cb.isChecked() )
        {
            vh.cb.setChecked( true );
            checkedCount++;
        }
        else
            vh.cb.setChecked( false );
        //holder.text.setText( list.get(position).getName() );
        //holder.cb.setChecked( list.get(position).isSelected() );

        /* LayoutInflater inflater = (LayoutInflater) context.getSystemService( Context.LAYOUT_INFLATER_SERVICE );
        View rowView = inflater.inflate( R.layout.location_row, parent, false );

        TextView textView = (TextView) rowView.findViewById( R.id.list_location_text );
        CheckBox cb = (CheckBox) rowView.findViewById( R.id.list_location_cb );
        LocationRecord record = getItem( position );
        textView.setText( record.toString() ); */

        Log.d( TAG, "Data: " + vh.text.getText() +
                " (" + ( vh.cb.isChecked() ? "Checked)" : ")" ) );

        Log.d( TAG, "getView( " + position + " ) = " + record.toDisplayString() );
        return rowView;
    }

    public void selectAll( boolean checked )
    {
        Log.d( TAG, "selectAll( " + checked + " )" );
        Log.d( TAG, "Total Items: " + list.size() + ", Checked: " + checkedCount );
        checkedCount = 0;
        for( int pos = 0; pos < list.size(); pos++ )
        {
            ViewHolder holder = getViewHolder( checkboxHash.get( pos ) );
            holder.cb.setChecked( checked );
        }
    }

    public void setChecked( int pos, boolean checked )
    {
        Log.d( TAG, "setChecked( " + pos + ", " + checked + " )" );
        ViewHolder holder = getViewHolder( checkboxHash.get( pos ) );
        holder.cb.setChecked( checked );
        checkedCount += ( checked ? 1 : -1 );
        Log.i( TAG, "setChecked() = " + holder.cb.isChecked() );
    }

    public void setChecked( int pos )
    {
        Log.d( TAG, "setChecked( " + pos + " )" );
        ViewHolder holder = getViewHolder( checkboxHash.get( pos ) );
        holder.cb.setChecked( !holder.cb.isChecked() );
        checkedCount += ( holder.cb.isChecked() ? 1 : -1 );
        Log.i( TAG, "setChecked() = " + holder.cb.isChecked() );
    }

    public ArrayList<LocationRecord> deleteItems()
    {
        Log.d( TAG, "deleteItems()" );

        ArrayList<LocationRecord> newList = new ArrayList<>();
        for( int pos = 0; pos < list.size(); pos++ )
        {
            ViewHolder holder = getViewHolder( checkboxHash.get( pos ) );
            if( holder.cb.isChecked() )
            {
                Log.i(TAG, "Deleting Record [" + pos + "]: " + list.get(pos));
                checkboxHash.remove( pos );
            }
            else
            {
                newList.add( list.get(pos) );
            }
        }

        Log.i( TAG, newList.size() + " items uanffected" );
        return newList;
    }

/*    public ArrayList<LocationRecord> deleteItems( ArrayList<Integer> items )
    {
        Log.d( TAG, "deleteItems( " + items.size() + " )" );
        if( items.size() == 0 )
            return null;

        ArrayList<LocationRecord> newList = new ArrayList<>();
        for( int pos = 0; pos < items.size(); pos++ )
        {
            Log.i( TAG, "Deleting Record [" + pos + "]: " + list.get(pos).toString() );
            newList.add( list.get( pos ) );
        }

        return newList;
    } */

    public ArrayList<Integer> getSelectedItems()
    {
        Log.d( TAG, "getSelectedItems()" );

        ArrayList<Integer> items = new ArrayList<>();
        for( int pos = 0; pos < list.size(); pos++ )
        {
            ViewHolder holder = getViewHolder( checkboxHash.get( pos ) );
            if( holder.cb.isChecked() )
            {
                Log.d( TAG, "Item " + list.get( pos ) + " [" + pos + "] is checked" );
                items.add( pos );
            }
        }

        Log.i( TAG, "Returning " + items.size() + " selected items" );
        return items;
    }

    public int getSelectedItemCount()
    {
        Log.d( TAG, "getSelectedItemCount()" );
        ArrayList<Integer> items = getSelectedItems();
        return items.size();
    }

    public LocationRecord getSelectedRecord()
    {
        Log.d( TAG, "getSelectedRecord()" );
        ArrayList<Integer> al = getSelectedItems();
        return list.get( al.get( al.size() - 1 ) );
    }

    public ArrayList<LocationRecord> replaceRecord( LocationRecord record )
    {
        Log.d( TAG, "replaceRecord( " + record + " )" );
        int pos = getSelectedItems().get( 0 );
        Log.d( TAG, "Replacing " + pos + " (" + list.get( pos ) + ")..." );
        list.remove( pos );
        list.add( pos, record );
        return list;
    }
}
