package com.tripgis.mobile.data;

import android.location.Location;
import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

import com.tripgis.mobile.FileManager;
import com.tripgis.mobile.utils.TripUtilities;
import com.google.android.gms.maps.LocationSource;
import com.google.android.gms.maps.model.LatLng;

import java.io.IOException;
import java.sql.Date;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.TimeZone;

/**
 * LocationRecord is the data of import being collected by the Android App.
 *
 * Created by pk on 7/9/17.
 */

public class TripRecord implements Parcelable
{
    private String name;                                // Title
    private ArrayList<LocationRecord> savedLocations;   // Stored items

    //TODO: Add additional map_items as they become known

    // Name to use in log lines
    public static final String TAG = TripRecord.class.getSimpleName();

    // Number of fields expected in a given record
    public static final int REQUIRED_FIELDS = 1;

    // Total number of fields in each record
    //   Used when writing records to pad each stored record with null fields
    public static final int TOTAL_NUM_FIELDS = REQUIRED_FIELDS + 0;

    public static final TripRecord EMPTY = new TripRecord( "" );

    // Constructor
    public TripRecord( String name )
    {
        Log.d( TAG, "TripRecord( " + name + " )" );
        this.name = name;

        // TODO force initial population
//        this.savedLocations = new ArrayList<>();
    }

    public String getName() { return name; }

    public String getStartTime()
    {
        LocationRecord record = getFirstRecord();
        if( record != null )
            return LocationRecord.getDateString( record.getDate() );

        return "";
    }

    public String getEndTime()
    {
        LocationRecord record = getLastRecord();
        if( record != null )
        {
            String text = LocationRecord.getDateString( record.getEndTime() );
            if( text.equals( "" ) )
                return LocationRecord.getDateString( record.getDate() );
        }

        return "";
    }

    // Setter methods
    public void setName( String n )     { name = n; }
    public LocationRecord getFirstRecord()
    {
        if( getItemCount() > 0 )
            return savedLocations.get( 0 );
        return LocationRecord.EMPTY;
    }

    public LocationRecord getLastRecord()
    {
        if( getItemCount() > 0 )
            return savedLocations.get( savedLocations.size() - 1 );
        return LocationRecord.EMPTY;
    }

    public int getItemCount() { return savedLocations.size(); }
    public ArrayList<LocationRecord> getSavedLocations() { return savedLocations; }
    public void setSavedLocations( ArrayList<LocationRecord> list )
    {
        savedLocations = list;
        TripUtilities.sortLocationRecords( savedLocations );
    }

    @Override
    public int describeContents()
    {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel out, int flags)
    {
        Log.d( TAG, "Parcel Out: " + this );
        out.writeString( name );
        out.writeInt( savedLocations.size() );
        for( LocationRecord record : savedLocations )
            out.writeParcelable( record, flags );
    }

    public static final Creator<TripRecord> CREATOR
            = new Creator<TripRecord>() {
        public TripRecord createFromParcel(Parcel in) {
            return new TripRecord(in);
        }

        public TripRecord[] newArray(int size) {
            return new TripRecord[size];
        }
    };

    private TripRecord(Parcel in)
    {
        name = in.readString();
        int size = in.readInt();
        LocationRecord[] array = LocationRecord.CREATOR.newArray(size);
        savedLocations = new ArrayList<>();
        for( int idx = 0; idx < size; idx++ )
            savedLocations.add( array[idx] );

        Log.d( TAG, "Parcel In: " + this );
    }

    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append( getName() );
        return sb.toString();
    }
}
