package com.tripgis.dmt.file;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import com.tripgis.dmt.types.EventRecord;
import com.tripgis.dmt.types.TripDbRecord;
import com.tripgis.dmt.util.DmtUtils;
import com.tripgis.dmt.util.LogTranslator;

/**
 * References:
 *    https://developer.android.com/guide/topics/data/data-storage.html#filesInternal
 *    https://docs.oracle.com/javase/7/docs/api/java/io/File.html
 *    https://www.mkyong.com/java/how-to-export-data-to-csv-file-java/
 *
 * Created by pk on 7/3/17.
 */

public class EventFileManager extends FileManager<TripDbRecord,EventRecord>
{
    // Name to use in log lines
    public static final String TAG = EventFileManager.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

    public EventFileManager( String name )
    { super( null, stripBasePostfix( name ) ); }
    public EventFileManager( File original ) { super( original ); }
    public EventFileManager( File original, String name )
    { super( original, name ); }
    public EventFileManager( File original, String name, boolean forceRead )
    { super( original, name, forceRead ); }

	@Override
	public String getExtension() { return '.' + DmtUtils.bmk; }

	@Override
	protected EventRecord itemToEvent( EventRecord e ) 
	{
		return e;
	}

	@Override
	protected EventRecord eventToItem( EventRecord e )
	{
		return e;
	}

	@Override
	protected TripDbRecord readHeader( BufferedReader lineReader ) throws IOException
	{
		Log.t( TAG, "readHeader( " + lineReader + " )" );
		String[] data = super.parseLine( lineReader );
		TripDbRecord record = TripDbRecord.buildFromCsvArray( data ); 
		record.setFileName( DmtUtils.removeExtension( originalFile.getName() ) );
		return record;
	}

	@Override
	protected EventRecord readListItem( String[] data ) throws IOException
	{
		Log.t( TAG, "readCsvString( " + data.length + " )" );
		return EventRecord.buildFromCsvArray( data );
	}

	@Override
	protected boolean writeHeader( FileWriter writer, TripDbRecord h ) throws IOException
	{
		writer.append( h.generateCsvString() + "\n" );
		return true;
	}

	@Override
	protected String getListItemString( EventRecord e )
	{
		return e.generateCsvString();
	}
}
