package com.tripgis.dmt.file;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import com.tripgis.dmt.types.EventRecord;
import com.tripgis.dmt.util.DmtUtils;
import com.tripgis.dmt.util.LogTranslator;
import com.tripgis.shared.types.LocationRecord;
import com.tripgis.shared.types.TripRecord;

/**
 * References:
 *    https://developer.android.com/guide/topics/data/data-storage.html#filesInternal
 *    https://docs.oracle.com/javase/7/docs/api/java/io/File.html
 *    https://www.mkyong.com/java/how-to-export-data-to-csv-file-java/
 *
 * Created by pk on 7/3/17.
 */

public class TripFileManager extends FileManager<TripRecord,LocationRecord>
{
    // Name to use in log lines
    public static final String TAG = TripFileManager.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

    public TripFileManager( String name )
    { super( null, stripBasePostfix( name ) ); }
    public TripFileManager( File original ) { super( original ); }
    public TripFileManager( File original, String name )
    { super( original, name ); }
    public TripFileManager( File original, String name, boolean forceRead )
    { super( original, name, forceRead ); }

    /**
     * References:
     *    https://stackoverflow.com/questions/8646984/how-to-list-files-in-an-android-directory
     *    https://stackoverflow.com/questions/924394/how-to-get-the-filename-without-the-extension-in-java
     *
     * @return
     * @throws IOException
     */
    public static ArrayList<String> getAvailableTrips() throws IOException
    {
        Log.d( TAG, "getAvailableTrips()" );
        File path = FileManager.getAlbumStorageDir();
        Log.d( TAG, "Reading " + path + "..." );

        ArrayList<String> trips = new ArrayList<>();
        File[] files = path.listFiles();
        if( files != null )
        {
            Log.d( TAG, "Number of files: " + files.length );
            for( int i = 0; i < files.length; i++ )
            {
                String name = files[i].getName();
                trips.add( name.replaceFirst("[.][^.]+$", "" ) );
                Log.d( TAG, "Adding File '" + files[i].getName() + "'" );
            }
        }

        return trips;
    }

//	@Override
//	public String getExtension() { return '.' + DmtUtils.csv; }

	@Override
	protected EventRecord itemToEvent( LocationRecord t ) 
	{
		return new EventRecord( t );
	}

	@Override
	protected LocationRecord eventToItem( EventRecord e )
	{
		return e.getMobileData();
	}

	@Override
	protected TripRecord readHeader( BufferedReader lineReader ) throws IOException
	{
		Log.t( TAG, "readHeader( " + lineReader + " )" );
		String[] data = super.parseLine( lineReader );
		TripRecord record = TripRecord.buildFromCsvArray( data ); 
		record.setFileName( DmtUtils.removeExtension( originalFile.getName() ) );
		return record;
	}

	@Override
	protected EventRecord readListItem( String[] data ) throws IOException
	{
		Log.t( TAG, "readCsvString( " + data.length + " )" );
		return itemToEvent( LocationRecord.buildFromCsvArray( data ) );
	}

	@Override
	protected boolean writeHeader( FileWriter writer, TripRecord h ) throws IOException
	{
		// NOTE: overWriteOriginal means nothing here
		
		writer.append( h.generateCsvString() + "\n" );
		return true;
	}

	@Override
	protected String getListItemString( EventRecord e )
	{
		if( overWriteOriginal )
			return e.getMobileData().generateCsvString();
		
		return e.generateCsvString();
	}
}
