package com.tripgis.dmt.types;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import com.tripgis.dmt.DmtDataType;
import com.tripgis.dmt.file.FileManager;
import com.tripgis.dmt.util.LogTranslator;

public class EventSubType implements Serializable, DmtDataType<EventSubType>
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -4170090806565260005L;

	// Name to use in log lines
    public static final String TAG = EventSubType.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );
	
	public static final EventSubType EMPTY = new EventSubType();
	public static final EventSubType NEW = new EventSubType( EventType.NEW, "NEW" );
	public static final ArrayList<EventSubType> EMPTY_LIST = new ArrayList<>();

    // Number of fields expected in a given record
    public static final int REQUIRED_FIELDS = 2;

	private EventType type;
	private String name;

	private EventSubType()
	{
		type = EventType.EMPTY;
		name = "";
	}
	
	public EventSubType( EventType e, String n )
	{
		type = e;
		name = n;
	}
	
	public EventSubType copy() { return new EventSubType( type.copy(), name ); }
	
	public EventType getType() { return type; }
	public String getName() { return name; }
	
	public static EventSubType buildFromCsvArray( String[] data ) throws IOException
	{
		Log.t( TAG, "buildFromCsvArray( " + data.length + " )" );

        // Verify the string array passed in has the correct number of fields
        if( data.length != REQUIRED_FIELDS )
            throw new IOException( "Invalid number of parameters (" + data.length + " vs. " + REQUIRED_FIELDS + ")" );

        Log.d( TAG, "Creating record from " + "'" + data[0] + "', '" + data[1] + "'..." );
        return new EventSubType( new EventType( data[0] ), data[1] );
	}

	@Override
	public boolean matches( EventSubType t )
	{
		return type.getName().equals( t.type.getName() ) && t.name.equals( name );
	}

	@Override
	public String generateCsvString() 
	{
		StringBuffer sb = new StringBuffer();
		sb.append( type + FileManager.CSV_SEPARATOR );
		sb.append( FileManager.prepareString( name ) );
		return sb.toString();
	}

	public String toString() 
	{ 
		StringBuffer sb = new StringBuffer();
		sb.append( "Type=" + type + "," );
		sb.append( "Name=" + name );
		return sb.toString();
	}
}
