package com.tripgis.dmt.ui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.types.EventRecordDisplayItem;
import com.tripgis.dmt.util.LogTranslator;

public class AnalysisForm extends JFrame 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 2954091012552735067L;

	// Name to use in log lines
    public static final String TAG = AnalysisForm.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

	private ButtonGroup rgAction;
	private JButton     btnOK;
	private JButton     btnCancel;
	
	private static final String UPDATE_DURATION    = "Update Duration";
//	private static final String ASSIGN_END_TIME    = "Assign End Time";
//	private static final String USE_GEO_LATLNG     = "Use Geocoder LatLngs";
//	private static final String USE_MOBILE_LATLNG  = "Use Mobile LatLngs";
//	private static final String PARSE_ADDRESS      = "Parse Address Data";
	private static final String ANALYZE            = "Analyze";
	private static final String GAP_STATS          = "Gap Statistics";
	
	private static DmtController controller;
	
	public AnalysisForm()
	{
		super( "Analysis Wizard" );
		Log.t( TAG, "AnalysisForm()" );
		
		controller = DmtController.getInstance();
		initializeComponents();
		
		setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
	}
	
	public void display()
	{
		Log.t( TAG, "display()" );
		setVisible( true );
	}
	
	public void close()
	{
		Log.t( TAG, "close()" );
		controller.closeForm( this );
		dispose();
	}
	
	/**
	 * References:
	 *    http://www.java.achchuthan.org/2011/06/in-this-section-you-will-learn-how-to.html
	 *    https://stackoverflow.com/questions/12838978/how-to-surround-java-swing-components-with-border
	 *    https://docs.oracle.com/javase/tutorial/displayCode.html?code=https://docs.oracle.com/javase/tutorial/uiswing/examples/components/DialogDemoProject/src/components/CustomDialog.java
	 */
	private void initializeComponents()
	{
		Log.t( TAG, "initializeComponents()" );
		Container c = getContentPane();
//		c.setLayout( new FlowLayout() );
		//setSize( 300, 300 );
	
		rgAction = new ButtonGroup();
		String[] actionList = new String[]{
				UPDATE_DURATION, //ASSIGN_END_TIME, USE_GEO_LATLNG,
				//USE_MOBILE_LATLNG, PARSE_ADDRESS, 
				ANALYZE, GAP_STATS
		};
		
		JPanel actionPanel = new JPanel();
		actionPanel.setLayout( new GridLayout( 0, 1 ) );
		actionPanel.setOpaque( true );
		//actionPanel.setBackground( Color.WHITE );
		actionPanel.setBorder(
				BorderFactory.createTitledBorder( "Select Action" )
		);
	
		ActionRadioHandler handler = new ActionRadioHandler( this );
		for( String text : actionList )
		{
			JRadioButton radio = new JRadioButton( text );
			actionPanel.add( radio );
			rgAction.add( radio );
		}

		JPanel buttonPanel = new JPanel();
		btnOK = new JButton( "OK" );
		btnCancel = new JButton( "Cancel" );
		
		btnOK.addActionListener( handler );
		btnCancel.addActionListener( handler );
		buttonPanel.add( btnOK );
		buttonPanel.add( btnCancel );
		
		c.add( actionPanel, BorderLayout.CENTER );
		c.add( buttonPanel, BorderLayout.SOUTH );
		pack();
	}
	
	class ActionRadioHandler implements ActionListener
	{
		private JFrame frame;
		
		public ActionRadioHandler( JFrame frame ) 
		{
			this.frame = frame;
		}

		@Override
		public void actionPerformed(ActionEvent e) 
		{
			Log.t( TAG, "actionPerformed( " + e + " )" );
			JButton button = (JButton) e.getSource();
			
			if( button.getText().equals( "Cancel" ) )
			{
				close();
				return;
			}

			String action = getSelectedRadioButton( rgAction );
			if( action == null )
			{
				DmtController.setError( TAG, "No action selected!" );
				return;
			}

			Log.d( TAG, "Action: " + action );
			switch( action )
			{
				case UPDATE_DURATION:
					controller.updateDurations();
					break;
//				case ASSIGN_END_TIME:
//					controller.assignEndTimes();
//					break;
				case ANALYZE:
					controller.generateStatistics();
					break;
				case GAP_STATS:
					ComboDialog<EventRecordDisplayItem> cd = new ComboDialog<>( DmtController.getDisplay().getEventCombo() );
					if( cd.showComboDialog( frame ) )
					{
						controller.generateGapStatistics( 
								cd.getSelect1().getRecord(),
								cd.getSelect2().getRecord() 
						);
					}
					break;
				default:
					DmtController.setError( TAG, "Unrecognized action - " + action );
					return;
			};

			close();
		}
	
		/**
		 * References:
		 *    https://stackoverflow.com/questions/201287/how-do-i-get-which-jradiobutton-is-selected-from-a-buttongroup
		 *    
		 * @param buttonGroup
		 * @return
		 */
		private String getSelectedRadioButton( ButtonGroup buttonGroup ) 
		{
			for( Enumeration<AbstractButton> buttons = buttonGroup.getElements(); buttons.hasMoreElements(); )
			{
				AbstractButton button = buttons.nextElement();
				
				if( button.isSelected() ) 
					return button.getText();
			}
			return null;
		}
	}

}
