package com.tripgis.dmt.ui;

import java.awt.GridLayout;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.file.TripFileManager;
import com.tripgis.dmt.util.LogTranslator;

class ComboDialog<T>
{
	// Name to use in log lines
    public static final String TAG = ComboDialog.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

	private String key;
	private JPanel panel;
	private JComboBox<T> combo1 = null;
	private JComboBox<T> combo2 = null;
	private JTextField text = null;
	private String userResponse;
	private T select1;
	private T select2;
	private boolean cancelled;
	
	public ComboDialog( String name, JComboBox<T> item )
	{
    	Log.t( TAG, "ComboDialog( " + name + ", " + item.getItemCount() + " )" );
    	key = name;
    	panel = new JPanel();
    	panel.setLayout( new GridLayout( 0, 2 ) );

    	JLabel clabel = new JLabel( "Current " + name );
    	JLabel nlabel = new JLabel( "New " + name  );
    	combo1 = new JComboBox<>();
    	text = new JTextField( 20 );

    	for( int i = 2; i < item.getItemCount(); i++ )
    	{
    		Log.d( TAG, "Adding " + item.getItemAt( i ) );
    		combo1.addItem( item.getItemAt( i ) );
    	}

    	combo1.setSelectedIndex( item.getSelectedIndex() - 2 );
    	panel.add( clabel );
    	panel.add( combo1 );
    	panel.add( nlabel );
    	panel.add( text );
	}

	public ComboDialog( JComboBox<T> item )
	{
    	Log.t( TAG, "ComboDialog( " + item.getItemCount() + " )" );
    	key = "Event";
    	panel = new JPanel();
    	panel.setLayout( new GridLayout( 0, 2 ) );

    	JLabel clabel = new JLabel( "First " + key );
    	JLabel nlabel = new JLabel( "Second " + key  );
    	combo1 = new JComboBox<>();
    	combo2 = new JComboBox<>();

    	for( int i = 2; i < item.getItemCount(); i++ )
    	{
    		Log.d( TAG, "Adding " + item.getItemAt( i ) );
    		combo1.addItem( item.getItemAt( i ) );
    		combo2.addItem( item.getItemAt( i ) );
    	}

    	combo1.setSelectedIndex( 0 );
    	combo2.setSelectedIndex( item.getItemCount() - 3 );
    	panel.add( clabel );
    	panel.add( combo1 );
    	panel.add( nlabel );
    	panel.add( combo2 );
	}

	public boolean showComboDialog( JFrame frame )
    {
    	String[] options = new String[]{ "OK", "Cancel" };
    	int option = JOptionPane.showOptionDialog( frame, panel, "The title",
    	                         JOptionPane.NO_OPTION, JOptionPane.PLAIN_MESSAGE,
    	                         null, options, options[1] );

    	cancelled = ( option != 0 );
    	boolean okResponse = ( option == 0 ); // pressing OK button
    	if( okResponse )
    	{
    		int count = setSelectFields();
    		if( count == 1 && text != null )
    			okResponse = validateEntry();
    		else if( count != 2 )
    		{
    			Log.d( TAG, "must select two items" );
    			okResponse = false;
    		}
    	}
    	
    	return okResponse;
    }
	
	private boolean validateEntry()
	{
		Log.t( TAG, "validateEntry()" );
		userResponse = text.getText();
		for( int i = 0; i < combo1.getItemCount(); i++ )
		{
			Log.d( TAG, "Comparing '" + userResponse + "' to '" +
					combo1.getItemAt(i) + "'." );
			if( userResponse.equals( combo1.getItemAt( i ).toString() ) )
			{
				userResponse = key + " already exists in DB";
				return false;
			}
		}
		
		if( key.equals( "Trip" ) )
		{
			Log.d( TAG, "Checking against files..." );
			try 
			{
				ArrayList<String> trips = TripFileManager.getAvailableTrips();
				for( String s : trips )
				{
					Log.d( TAG, "Comparing '" + userResponse + "' to '" + s + "'" );
					if( userResponse.equals( s ) )
					{
						userResponse = key + " has a mobile data file";
						return false;
					}
				}
			} 
			catch (IOException e) 
			{
				DmtController.setError( TAG, "Could not obtain trip list to verify, caution!" );
			}
		}
		
		return true;
	}
	
	@SuppressWarnings("unchecked")
	private int setSelectFields()
	{
		select1 = (T) combo1.getSelectedItem();
		if( combo2 == null )
			return 1;
		
		select2 = (T) combo2.getSelectedItem();
		return 2;
	}
	
	public boolean isCancelled() { return cancelled; }
	public String getReason()    { return userResponse; }
	public String getUserText()  { return userResponse; }
	public T getSelect1()        { return select1; }
	public T getSelect2()        { return select2; }
}
