package com.tripgis.dmt.ui;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.filechooser.FileFilter;

import com.tripgis.dmt.util.DmtUtils;

class CustomFileChooserFilter extends FileFilter
{
	private String filterText;
	private boolean filterTextSet = false;
	private List<String> extensions;
	private String regex;
	
	//http://docs.oracle.com/javase/tutorial/uiswing/components/filechooser.html
	
	public CustomFileChooserFilter()
	{
		filterText = "Image Files (.tiff, .jpg, .png, .gif)";
		extensions = new ArrayList<String>();
		regex = ".*";
		extensions.add( DmtUtils.jpeg );
		extensions.add( DmtUtils.jpg );
		extensions.add( DmtUtils.gif );
		extensions.add( DmtUtils.tiff );
		extensions.add( DmtUtils.tif );
		extensions.add( DmtUtils.png );
	}
	
	public CustomFileChooserFilter( String regex )
	{
		filterText = "Matched Files (Any)";
		extensions = new ArrayList<String>();
		regex = ".*";

		setMatchFilter( regex );
	}

	public CustomFileChooserFilter( List<String> list )
	{
		filterText = "Related Files (";
		extensions = new ArrayList<String>();
		regex = ".*";

		for( String item : list )
		{
			filterText += item + ", ";
			extensions.add( item );
		}

		filterText = filterText.substring( 0, filterText.length() - 2 );
		filterText += ")";
		chopText();
	}
	
	public void setFilterText( String text )
	{
		filterText = text;
		filterTextSet = true;
	}

	public void setMatchFilter( String regex )
	{
		this.regex = regex;
		chopText();
	}
	
	private void addRegexText()
	{
		filterText = filterText.substring( 0, filterText.length() - 1 );
		filterText += regex.substring( 0, 10 );
		if( regex.length() > 10 )
			filterText += "...";
		filterText += ")";
	}
	
	@Override
	public boolean accept( File f )
	{
		if( f.isDirectory() )
			return true;
		
		String extension = DmtUtils.getExtension( f );
		if( extension != null )
			if( extensions.contains( extension ) )
				return extension.matches( regex );
		
		return false;
	}
	
	@Override
	public String getDescription() { return filterText; }
	
	private void chopText()
	{
		if( !filterTextSet && filterText.length() > 25 )
		{
			filterText = filterText.substring( 0, filterText.indexOf( '(' ) );
			filterText += extensions.size() + " Extensions, )";
			if( regex != ".*" )
				addRegexText();
		}
	}
}
