package com.tripgis.dmt.ui;

import java.awt.Container;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import java.sql.Date;
import java.text.ParseException;
import java.util.ArrayList;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.file.FileManager;
import com.tripgis.dmt.geocode.Geocoder;
import com.tripgis.dmt.types.DistanceRecord;
import com.tripgis.dmt.types.EventRecord;
import com.tripgis.dmt.types.EventRecordDisplayItem;
import com.tripgis.dmt.types.EventSubType;
import com.tripgis.dmt.types.EventSubTypeDisplayItem;
import com.tripgis.dmt.types.EventType;
import com.tripgis.dmt.types.EventTypeDisplayItem;
import com.tripgis.dmt.types.MatchRecord;
import com.tripgis.dmt.types.TripDbRecord;
import com.tripgis.dmt.types.TripDbRecordDisplayItem;
import com.tripgis.dmt.ui.listeners.EventButtonActionListener;
import com.tripgis.dmt.ui.listeners.EventEnabler;
import com.tripgis.dmt.ui.listeners.EventFormListener;
import com.tripgis.dmt.ui.listeners.EventFormSelectionListener;
import com.tripgis.dmt.ui.listeners.EventKeyListener;
import com.tripgis.dmt.ui.listeners.ProgramCloseHandler;
import com.tripgis.dmt.util.DmtProperties;
import com.tripgis.dmt.util.ExternalProgram;
import com.tripgis.dmt.util.LogTranslator;
import com.tripgis.shared.types.LocationRecord;

public class EventForm extends WindowAdapter implements
	EventEnabler, ProgramCloseHandler
{
	private JFrame frame;
	private JTabbedPane tabbedPane;
	private JPanel eventPane;
	private JPanel controlPane;
	
	// Support tabs with additional data fields
	private DmtController       controller;
	private EventFormMenu        eventMenu;
	private EventFormLocationTab locationTab;
	private EventFormNearbyTab   nearbyTab;
	private EventDialogs         dialogs;
//	private EventFormListener    listener;

	// Fields that can be modified by user
	private JComboBox<TripDbRecordDisplayItem> cbTrip;
	private JComboBox<EventRecordDisplayItem> cbEvent;
	private JTextField txtDate;
	private JComboBox<EventTypeDisplayItem> cbEventType;
	private JComboBox<EventSubTypeDisplayItem> cbEventSubType;
	private JTextField txtEndTime;
	private JTextArea txtNotes;

	// Control buttons
	private JButton btnSave;
	private JButton btnReset;
	private JButton btnDelete;
	private JButton btnClose;
	
	// Status message
	private JLabel msgLabel;

	private boolean eventChanged;
	private boolean tripChanged;

	// Name to use in log lines
    public static final String TAG = EventForm.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

    public static final String DEFAULT_TEXT = "- -";

    // Location Details Tab Buttons
	public static final String GEOCODE      = "Geocode";
	public static final String UPDATE_ASSOC = "Update";
	public static final String CHANGE_TRIP  = "Move Trip";
	public static final String SET_ASSOC    = "Assign";
	
	// Nearby Tab Buttons
	public static final String SEARCH  = "Search";
	public static final String REFRESH = "Refresh";
	
	// Control Buttons
	public static final String SAVE   = "Save";
	public static final String RESET  = "Reset";
	public static final String DELETE = "Delete";
	public static final String CLOSE  = "Close";
	public static final String EXIT  = " Exit";

	public static final String QGIS_PROGRAM = DmtProperties.getProperty( "gis.qgis.command" ); //"/usr/bin/qgis %F";
	public static final String ANALYZE_PROGRAM = "/usr/bin/XXX";
	
	public EventForm() 
	{
		Log.t( TAG, "EventForm()" );
		
		msgLabel = new JLabel( "Initializing Gui..." );

        eventMenu = new EventFormMenu();
		locationTab = new EventFormLocationTab();
		nearbyTab = new EventFormNearbyTab();
		
		//connectToDb = new ConnectToDbCommand();
		//listener = new EventFormListener();
    }

	public void initialize()
	{
		Log.t( TAG, "initialize()" );

		controller = DmtController.getInstance();
		eventMenu.initialize();
		locationTab.initialize();
		nearbyTab.initialize();
		
		initializeEvent();
		initializeTabbedPanels();
		initializeControls();
		initializeFrame();
		initializeComponents();
	}
	
	private void initializeTabbedPanels()
	{
		Log.t( TAG, "initializeTabbedPanels()" );

		tabbedPane = new JTabbedPane();
		tabbedPane.addTab( "Details", null, locationTab.getTab(), null );
		tabbedPane.setMnemonicAt( 0, KeyEvent.VK_1 );
		
		//locationTab.enableTab( false );
		//nearbyTab.enableTab( false );
		tabbedPane.addTab( "Search", null, nearbyTab.getTab(), null );
		tabbedPane.setMnemonicAt( 1, KeyEvent.VK_2 );
	}
	
	private void initializeFrame()
	{
		Log.t( TAG, "initializeFrame()" );
		
        // Create and set up the window.
        frame = new JFrame( "TripGIS - Data Management" );
        frame.setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
        frame.addWindowListener( this );
        frame.addKeyListener( new EventKeyListener() );

        frame.setJMenuBar( eventMenu );
//        frame.setResizable( false );
        dialogs = new EventDialogs( frame );
	}

	private void initializeComponents() 
	{    
		Log.t( TAG, "initializeComponents()" );

		SpringLayout layout = new SpringLayout();
		Container content = frame.getContentPane();
		content.add( eventPane );
		content.add( tabbedPane );
		content.add( controlPane );

		content.setLayout( layout );
		layout.putConstraint( SpringLayout.WEST, eventPane, 5, SpringLayout.WEST, content );
		layout.putConstraint( SpringLayout.WEST, tabbedPane, 5, SpringLayout.WEST, content );
		layout.putConstraint( SpringLayout.WEST, controlPane, 5, SpringLayout.WEST, content );
		layout.putConstraint( SpringLayout.NORTH, eventPane, 5, SpringLayout.NORTH, content );
		layout.putConstraint( SpringLayout.NORTH, tabbedPane, 5, SpringLayout.SOUTH, eventPane );
		layout.putConstraint( SpringLayout.NORTH, controlPane, 5, SpringLayout.SOUTH, tabbedPane );
		layout.putConstraint( SpringLayout.EAST, content, 5, SpringLayout.EAST, controlPane );
		layout.putConstraint( SpringLayout.EAST, content, 5, SpringLayout.EAST, eventPane );
//		layout.putConstraint( SpringLayout.EAST, content, 5, SpringLayout.EAST, tabbedPane );
		layout.putConstraint( SpringLayout.SOUTH, content, 5, SpringLayout.SOUTH, controlPane ); 
////		frame.setSize( 400, 700 );
	}
	
	/**
	 * References:
	 *    https://docs.oracle.com/javase/tutorial/uiswing/layout/visual.html
	 *    https://docs.oracle.com/javase/tutorial/uiswing/components/combobox.html
	 *    https://stackoverflow.com/questions/21658144/jpanel-components-change-position-automatically/21659516#21659516
	 */
	private void initializeEvent()
	{
		Log.t( TAG, "initializeEvent()" );

		eventPane = new JPanel();
		JPanel eventDetails = new JPanel();
		
		JLabel trip    = new JLabel( "Trip" );
		JLabel event   = new JLabel( "Event" );
		JLabel date    = new JLabel( "Date/Time" );
		JLabel type    = new JLabel( "Type" );
		JLabel subtype = new JLabel( "Subtype" );
		JLabel endtime = new JLabel( "EndTime" );
				
		JLabel[] labels = new JLabel[6];
		labels[0] = trip;
		labels[1] = event;
		labels[2] = date;
		labels[3] = type;
		labels[4] = subtype;
		labels[5] = endtime;

		cbTrip         = new JComboBox<>();
		cbEvent        = new JComboBox<>();
		txtDate        = new JTextField( 20 );
		cbEventType    = new JComboBox<>();
		cbEventSubType = new JComboBox<>();
		txtEndTime     = new JTextField( 20 );
		txtNotes       = new JTextArea( 2, 25 );
		
		JComponent[] fields = new JComponent[6];
		fields[0] = cbTrip;
		fields[1] = cbEvent;
		fields[2] = txtDate;
		fields[3] = cbEventType;
		fields[4] = cbEventSubType;
		fields[5] = txtEndTime;
		
		EventFormSelectionListener listener = new EventFormSelectionListener();
		cbTrip.addItemListener( listener );
		cbEvent.addItemListener( listener );
		cbEventType.addItemListener( listener );
		cbEventSubType.addItemListener( listener );

		GroupLayout layout = prepareGroupLayout( eventDetails, labels, fields );		
		eventDetails.setLayout( layout );
		layout.setAutoCreateGaps( true );

		JLabel notes = new JLabel( "Notes" );
		JScrollPane scroll = new JScrollPane( txtNotes );

		eventPane.add( eventDetails );
		eventPane.add( notes );
		eventPane.add( scroll );

		SpringLayout eLayout = new SpringLayout();
		eventPane.setLayout( eLayout );
		eLayout.putConstraint( SpringLayout.WEST, notes, 5, SpringLayout.WEST, eventPane );
		eLayout.putConstraint( SpringLayout.WEST, eventDetails, 5, SpringLayout.WEST, eventPane );
		eLayout.putConstraint( SpringLayout.WEST, scroll, 5, SpringLayout.WEST, eventPane );
		eLayout.putConstraint( SpringLayout.NORTH, eventDetails, 5, SpringLayout.NORTH, eventPane );
		eLayout.putConstraint( SpringLayout.NORTH, notes, 5, SpringLayout.SOUTH, eventDetails );
		eLayout.putConstraint( SpringLayout.NORTH, scroll, 5, SpringLayout.SOUTH, notes );
//		eLayout.putConstraint( SpringLayout.EAST, eventPane, 5, SpringLayout.EAST, eventDetails );
		eLayout.putConstraint( SpringLayout.EAST, eventPane, 5, SpringLayout.EAST, scroll );
//		eLayout.putConstraint( SpringLayout.EAST, eventPane, 5, SpringLayout.EAST, notes );
		eLayout.putConstraint( SpringLayout.SOUTH, eventPane, 5, SpringLayout.SOUTH, scroll );
	}
	
	public static GroupLayout prepareGroupLayout( JPanel panel, JLabel[] labels, JComponent[] fields )
	{
		Log.t( TAG, "prepareGroupLayout( <JPanel>, JLabel[], JComponent[] )" );

		GroupLayout layout = new GroupLayout( panel );
		GroupLayout.SequentialGroup hGroup = layout.createSequentialGroup();
		GroupLayout.Group labelGroup = layout.createParallelGroup( GroupLayout.Alignment.TRAILING );
		hGroup.addGroup( labelGroup );
		GroupLayout.Group fieldGroup = layout.createParallelGroup();
		hGroup.addGroup( fieldGroup );
		layout.setHorizontalGroup( hGroup );
		GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();
		layout.setVerticalGroup( vGroup );

		int p = GroupLayout.PREFERRED_SIZE;
		for (int i = 0; i < labels.length; i++) 
		{
			labelGroup.addComponent( labels[i] );
			fieldGroup.addComponent( fields[i] );

			vGroup.addGroup( layout.createParallelGroup()
                    .addComponent( labels[i] )
                    .addComponent( fields[i], p, p, p ) );
        }
		
		return layout;
	}

	private void initializeControls()
	{
		Log.t( TAG, "initializeControls()" );
		controlPane = new JPanel();
		JPanel controls = new JPanel();
		
		btnSave   = new JButton( SAVE );
		btnReset  = new JButton( RESET );
		btnDelete = new JButton( DELETE );
		btnClose  = new JButton( CLOSE );
		
		EventButtonActionListener listener = new EventButtonActionListener();
		btnSave.addActionListener( listener );
		btnReset.addActionListener( listener );
		btnDelete.addActionListener( listener );
		btnClose.addActionListener( listener );

		controls.add( btnSave );
		controls.add( btnReset );
		controls.add( btnDelete );
		controls.add( btnClose );
		
		controlPane.add( controls );
		controlPane.add( msgLabel );

		SpringLayout eLayout = new SpringLayout();
		controlPane.setLayout( eLayout );
		eLayout.putConstraint( SpringLayout.WEST, controls, 5, SpringLayout.WEST, controlPane );
		eLayout.putConstraint( SpringLayout.WEST, msgLabel, 5, SpringLayout.WEST, controlPane );
		eLayout.putConstraint( SpringLayout.NORTH, controls, 5, SpringLayout.NORTH, controlPane );
		eLayout.putConstraint( SpringLayout.NORTH, msgLabel, 5, SpringLayout.SOUTH, controls );
		eLayout.putConstraint( SpringLayout.EAST, controlPane, 5, SpringLayout.EAST, controls );
		eLayout.putConstraint( SpringLayout.SOUTH, controlPane, 5, SpringLayout.SOUTH, msgLabel );
	}

    public void display() 
    {
		Log.t( TAG, "display()" );
		enableOnFormCreate();

		//Display the window.
//    	frame.setSize( 1024, 768 );
    	frame.setSize( 350, 750 );
//        frame.pack();
    	frame.setResizable( false );
        frame.setVisible(true);
    }

	public void enableListeners( boolean flag )
	{
		Log.t( TAG, "enableListeners( " + flag + " )" );
		EventFormListener.setActive( flag );
	}
	
    public JFrame getFrame()                     { return frame;       }
    public EventFormMenu getMenu()               { return eventMenu;   }
    
    public JTabbedPane getTabbedPane()           { return tabbedPane;  }
    public EventFormLocationTab getLocationTab() { return locationTab; }
    public EventFormNearbyTab getNearbyTab()     { return nearbyTab;   }
    public EventDialogs getDialogs()             { return dialogs;     }
    
    public JComboBox<EventRecordDisplayItem> getEventCombo() { return cbEvent;     }
    public JButton getSaveButton()               { return btnSave;     }
    
    public boolean isEventChanged() { return eventChanged; }
    public boolean isTripChanged() { return tripChanged; }

    public void setChanged( boolean c ) 
	{ 
		eventChanged = c;
		tripChanged = c;
	}
    
    public void setMessage( String tag, String message )
    {
    	Log.d( tag, "Message: " + message );
    	msgLabel.setText( message );
    }
    
    /*******************************************************
     * FORM COMBO BOX SELECTION GETTERS
     ******************************************************/
    
    public TripDbRecord getSelectedTrip()
    {
    	TripDbRecordDisplayItem tdi = (TripDbRecordDisplayItem) cbTrip.getSelectedItem();
    	if( tdi == null )
    	{
    		Log.d( TAG, "getSelectedTrip(): No Trip selected" );
    		return TripDbRecord.EMPTY;
    	}
    	return tdi.getRecord();
    }
    
    public EventRecord getSelectedEvent()
    {
    	EventRecordDisplayItem edi = (EventRecordDisplayItem) cbEvent.getSelectedItem();
    	if( edi == null )
    	{
    		Log.d( TAG, "getSelectedEvent(): No Event selected" );
    		return EventRecord.EMPTY;
    	}
    	return edi.getRecord();
    }
    
    public EventType getSelectedEventType()
    {
    	EventTypeDisplayItem edi = (EventTypeDisplayItem) cbEventType.getSelectedItem();
    	if( edi == null )
    	{
    		Log.d( TAG, "getSelectedEvent(): No EventType selected" );
    		return EventType.EMPTY;
    	}
    	return edi.getType();
    }
    
    public EventSubType getSelectedEventSubType()
    {
    	EventSubTypeDisplayItem edi = (EventSubTypeDisplayItem) cbEventSubType.getSelectedItem();
    	if( edi == null )
    	{
    		Log.d( TAG, "getSelectedEvent(): No EventSubType selected" );
    		return EventSubType.EMPTY;
    	}
    	return edi.getType();
    }
	
    /*******************************************************
     * FORM COMBO BOX SELECTORS
     ******************************************************/
    
    
	private void selectTrip( TripDbRecord t )
	{
		Log.t( TAG, "selectTrip( " + t + " )" );
		
		if( tripChanged )
			tripChanged = dialogs.yesNoDialog( "Changes will be lost. Do you wish to continue?", "Unsaved Data" );

		tripChanged = false;
		eventChanged = false;
		setEventList( t.getList() );
    	selectTripPart( t );
    	if( t == TripDbRecord.EMPTY )
    		setMessage( TAG, "Form Reset!" );
    	else
    		setMessage( TAG, "Trip " + t.getTripName() + " selected!" );
	}

	private void selectTripPart( TripDbRecord t )
	{
		Log.t( TAG, "selectTripPart( " + t + " )" );

		for( int i = 0; i < cbTrip.getItemCount(); i++ )
		{
			if( cbTrip.getItemAt( i ).getRecord() == t )
			{
				Log.d( TAG, "Matched [" + i + "]" );
		    	enableListeners( false );
				cbTrip.setSelectedIndex( i );
		    	enableListeners( true );
				break;
			}
		}
	}

	private void selectEvent( EventRecord e )
	{
		Log.t( TAG, "selectEvent( " + e + " )" );

		if( eventChanged && !isChangeDiscardable() )
		{
			setMessage( TAG, "Event update cancelled!" );
			return;
		}
		
		eventChanged = false;
		selectEventPart( e );
		txtDate.setText( LocationRecord.getSqlDateString( e.getMobileData().getDate() ) );

		selectEventType( e.getEventSubType().getType() );
		selectEventSubType( e.getEventSubType() );
		txtEndTime.setText( LocationRecord.getSqlDateString( e.getMobileData().getEndTime() ) );
		txtNotes.setText( displayNotes( e ) );
		parseNotes();
		locationTab.setEvent( e );

		nearbyTab.clear();
		enableOnEventSelected( e != EventRecord.EMPTY );
    	setMessage( TAG, "Event " + e.getDescription() + " selected!" );
	}

	private void selectEventPart( EventRecord e )
	{
		Log.t( TAG, "selectEventPart( " + e + " )" );
		for( int i = 0; i < cbEvent.getItemCount(); i++ )
		{
			if( cbEvent.getItemAt( i ).getRecord() == e )
			{
				Log.d( TAG, "Matched [" + i + "]" );
		    	enableListeners( false );
				cbEvent.setSelectedIndex( i );
		    	enableListeners( true );
				break;
			}
		}
	}

    public void selectEventType( EventType type )
    {
    	Log.t( TAG,  "selectEventType( " + type + " )" );
    	if( type == EventType.EMPTY || type.getName().isEmpty() )
    	{
			Log.d( TAG, "Selecting EMPTY record" );
	    	enableListeners( false );
			cbEventType.setSelectedIndex( 0 );
	    	enableListeners( true );
    	}

    	for( int i = 0; i < cbEventType.getItemCount(); i++ )
    	{
    		if( cbEventType.getItemAt( i ).getType() == type )
    		{
    			Log.d( TAG, "Matched " + cbEventType.getItemAt( i ).getType() );
    	    	enableListeners( false );
    			cbEventType.setSelectedIndex( i );
    	    	enableListeners( true );
    			break;
    		}
    	}

    	controller.updateEventSubTypeList( type );
    	selectEventSubType( EventSubType.EMPTY );
    	enableOnEventTypeSelected();
    	setMessage( TAG, "EventType " + type.getName() + " selected!" );
    }

    public void selectEventSubType( EventSubType type )
    {
    	Log.t( TAG,  "selectEventSubType( " + type + " )" );
    	Log.d( TAG, "Number of subtypes: " + cbEventSubType.getItemCount() );
    	for( int i = 0; i < cbEventSubType.getItemCount(); i++ )
    	{
    		Log.d( TAG, "Comparing " + cbEventSubType.getItemAt( i ).getType() + " to " + type );
    		if( cbEventSubType.getItemAt( i ).getType() == type )
    		{
    	    	enableListeners( false );
    			cbEventSubType.setSelectedIndex( i );
    	    	enableListeners( true );
    			break;
    		}
    	}

    	enableOnEventSubTypeSelected();
    	setMessage( TAG, "EventSubType " + type.getName() + " selected!" );
    }

    public EventRecord getEventDataFromForm()
    {
    	EventRecord er = getSelectedEvent().copy();
    	er.setTripId( getSelectedTrip().getTripId() );
    	er.setEventSubType( getSelectedEventSubType() );

    	String[] txt = parseNotes();
    	er.setDescription( txt[0] );
    	er.setNotes( txt[1] );

    	boolean dateParsed = false;
    	try 
		{
        	if( er.getEventId() != 0 )
        	{
    			Date date = LocationRecord.parseDate( txtDate.getText() );
    	    	er.getMobileData().setDate( date );
        	}

        	dateParsed = true;
    		if( !txtEndTime.getText().isEmpty() )
    		{
				Date endTime = LocationRecord.parseDate( txtEndTime.getText() );
		    	er.getMobileData().setEndTime( endTime );
    		}
		}
		catch( ParseException e )
		{
			if( dateParsed )
				DmtController.setError( TAG, "Failed to prepare Date - " + e );
			else
				DmtController.setError( TAG, "Failed to prepare End Time - " + e );
		}
		catch( NumberFormatException e )
		{
			if( dateParsed )
				DmtController.setError( TAG, "Failed to prepare Date - " + e );
			else
				DmtController.setError( TAG, "Failed to prepare End Time - " + e );
		}

		locationTab.getEventDataFromTab( er );
    	return er;
    }
    
    public boolean exitApplication()
    {
    	Log.t( TAG, "exitApplication()" );
		boolean close = true;
		
		TripDbRecord trip = getSelectedTrip();
		if( ( trip.isChanged() || eventChanged || tripChanged ) && !isChangeDiscardable() )
		{
			setMessage( TAG, "Exit cancelled" );
			close = false;
		}
		
		eventChanged = false;
		if( close )
			this.frame.dispose();

		return close;
    }
    
    @Override
	public void windowClosing( WindowEvent e ) 
	{ 
    	Log.t( TAG, "windowClosing( <WindowEvent> )" );
    	controller.exitProgram(); 
	}

    /*******************************************************
     * FORM COMBO BOX INITIALIZERS
     ******************************************************/
    
    public void setTripList( ArrayList<TripDbRecord> list )
    {
    	Log.t( TAG, "setTripList( " + list.size() + " )" );
    	enableListeners( false );
    	cbTrip.removeAllItems();
    	cbTrip.addItem( new TripDbRecordDisplayItem( TripDbRecord.EMPTY ) );
    	cbTrip.addItem( new TripDbRecordDisplayItem( TripDbRecord.NEW ) );
    	for( TripDbRecord record : list )
    		cbTrip.addItem( new TripDbRecordDisplayItem( record ) );
    	
    	Log.d( TAG, "Number of Trips: " + cbTrip.getItemCount() );
    	selectTrip( TripDbRecord.EMPTY );
    	enableListeners( true );
    }
        
    public void setEventList( ArrayList<EventRecord> list )
    {
    	Log.t( TAG,  "setEventList( " + list.size() + " )" );

    	enableListeners( false );
    	cbEvent.removeAllItems();
    	cbEvent.addItem( new EventRecordDisplayItem( EventRecord.EMPTY ) );
    	cbEvent.addItem( new EventRecordDisplayItem( EventRecord.NEW ) );

		//boolean failed = false;
    	for( int i = 0; i < list.size(); i++ )
    	{
    		EventRecord er = list.get( i );
    		EventRecordDisplayItem erdi = new EventRecordDisplayItem( er );
/*    		if( er.getMobileData().getEndTime() == null || er.getMobileData().getEndTime().getTime() == 0 )
    		{
	    			Log.d( TAG, "Event " + i + " does not have endtime, calculating..." );
	    			if( i + 1 >= list.size() )
	    			{
	    				Log.d( TAG, "Last event, no end time" );
	    	    		cbEvent.addItem( erdi );
	    				break;
	    			}
	
	    			DistanceRecord dr = DistanceRecord.EMPTY;
	    			EventRecord next = list.get( i + 1 );
	    			boolean matches = er.getMobileData().getLocation().equals( next.getMobileData().getLocation() );
	    			if( !failed  && i < 5 )
	    			{
		    			// Consider adding various options
		    			try
		    			{
		    				dr = DmtController.getGeocoder().getDistance( er, next );
		    				String status = DmtController.getGeocoder().getLastRequestStatus();
		    				if( status.equals( Geocoder.STATUS_OK ) )
		    					erdi.setDistance( dr );
		    				else
		    				{
			    				dr = DistanceRecord.EMPTY;
		    					DmtController.setError( TAG, "Last request failed, RC=" + status );
			    				failed = true;
		    				}
		    			}
		    			catch( IOException e )
		    			{
		    				DmtController.setError( TAG, "Failed to get distance for " + er + " - " + e );
		    				failed = true;
		    			}
	    			}
	    			
	    			if( failed && matches )
	    			{
	    				Log.d( TAG, "Locations match, using time - 1" );
	    				dr = DistanceRecord.EMPTY.clone();
	    				dr.setOrigin( er.getGeoData() );
	    				dr.setDestination( next.getGeoData() );
	    				dr.setDistance( 0 );
	    				dr.setTextDistance( "same" );
	    				dr.setDuration( 1 ); // use the next time minus a second
	    				dr.setTextDuration( "1 sec" );
	    			}

	    			if( dr != DistanceRecord.EMPTY )
	    			{
	    				long time = next.getMobileData().getDate().getTime() - dr.getDuration();
	    				er.getMobileData().setEndTime( new Date( time ) );
	    				er.setNotes( er.getNotes() + "\r\n<br>" + "Dur: " + dr.getTextDuration() + ", Dis: " + dr.getTextDistance() );
	    				Log.d( TAG, "Event " + i + " End Time: " + 
	    					LocationRecord.getDateString( er.getMobileData().getEndTime() ) );
	    			}
    		}*/

    		cbEvent.addItem( erdi );
    	}
    	
    	Log.d( TAG, "Number of Events: " + cbEvent.getItemCount() );
    	selectEvent( EventRecord.EMPTY );
    	enableListeners( true );
    }
        
    public void setEventTypeList( ArrayList<EventType> list )
    {
    	Log.t( TAG,  "setEventTypeList( " + list.size() + " )" );

    	enableListeners( false );
    	cbEventType.removeAllItems();
    	cbEventType.addItem( new EventTypeDisplayItem( EventType.EMPTY ) );
    	cbEventType.addItem( new EventTypeDisplayItem( EventType.NEW ) );
    	for( EventType type : list )
    	{
    		Log.d( TAG, "Adding " + type );
    		cbEventType.addItem( new EventTypeDisplayItem( type ) );
    	}

    	Log.d( TAG, "Number of Types: " + cbEventType.getItemCount() );
    	selectEventType( EventType.EMPTY );
    	enableListeners( true );
    }
    
    public void setEventSubTypeList( ArrayList<EventSubType> list )
    {
    	Log.t( TAG,  "setEventSubTypeList( " + list.size() + " )" );

    	enableListeners( false );
    	cbEventSubType.removeAllItems();
    	cbEventSubType.addItem( new EventSubTypeDisplayItem( EventSubType.EMPTY ) );
    	cbEventSubType.addItem( new EventSubTypeDisplayItem( EventSubType.NEW ) );
    	for( EventSubType type : list )
    	{
    		if( getSelectedEventType() == type.getType() )
    			cbEventSubType.addItem( new EventSubTypeDisplayItem( type ) );
    	}

    	Log.d( TAG, "Item Count: " + cbEventSubType.getItemCount() );
    	selectEventSubType( EventSubType.EMPTY );
    	enableListeners( true );
    }

    /*******************************************************
     * FORM COMBO BOX SELECTORS
     ******************************************************/
    
/*    public void setEventSubType( EventSubType type )
    {
    	Log.t( TAG,  "setEventSubType( " + type + " )" );
    	for( int i = 0; i < cbEventSubType.getItemCount(); i++ )
    	{
    		EventSubTypeDisplayItem item = (EventSubTypeDisplayItem) cbEventSubType.getItemAt( i );
    		Log.d( TAG, "compare " + item.getType() + " == " + type +
    				": " + ( item.getType() == type ) );
    		if( item.getType() == type )
    		{
    			enableListeners( false );
    			cbEventSubType.setSelectedIndex( i );
    			enableListeners( true );
    			break;
    		}
    	}
    } */

/*    public void setEmptyEventSubType( EventType type )
    {
    	Log.t( TAG,  "setEmptyEventSubType( " + type + " )" );
    	enableListeners( false );
    	controller.updateEventSubTypeList( type );
    	cbEventSubType.setSelectedIndex( 0 );
    	enableListeners( true );
    }*/

	public String displayNotes( EventRecord e )
	{
		Log.t( TAG, "displayNotes( " + e + " )" );
		if( e == EventRecord.EMPTY )
			return "";

		if( e.getDescription().length() > 1 )
		{
			Log.t( TAG, "setting notes via desc + notes" );
			return FileManager.readString( "[" + e.getDescription() + "]" + e.getNotes() );
		}

		Log.t( TAG, "setting notes via mobile data (no description)" );
		String text = e.getMobileData().getName();

		// Convert <CMA> to commas
		String conv = FileManager.readString( text );
		String[] parts = conv.split( "\\s+" );

		int idx = 0;
		String desc = "";
		while( idx < parts.length && desc.length() + parts[idx].length() < EventRecord.MAX_DISPLAY_LEN )
			desc += parts[idx++] + " ";

		String notes = "[" + desc + "]\r\n";
		for( ; idx < parts.length; idx++ )
			notes += parts[idx];

		return notes;
	}

    public String[] parseNotes()
    {
    	Log.t( TAG, "parseNotes()" );
    	String notes = txtNotes.getText();
    	String[] ret = new String[2];
    	ret[1] = "";

    	if( notes == null )
    	{
    		ret[0] = getSelectedEvent().getMobileData().getName();
    		Log.d( TAG, "Returning Name: " + ret[0] );
    		return ret;
    	}

    	if( notes.length() < 2 )	// At a minimum, notes = []
    	{
    		ret[0] = getSelectedEvent().getMobileData().getName();
    		ret[1] = notes;
    		Log.d( TAG, "Returning Name: " + ret[0] );
    		Log.d( TAG, "         Notes: " + ret[1] );
    		return ret;
    	}

    	int start = notes.indexOf( '[' );
    	int end;
    	
    	if( start < 0 )
    		start = -1;

    	end = notes.indexOf( ']', start + 1 );
    	if( end < 0 )
    		end = notes.length() - 1;

    	ret[0] = notes.substring( start + 1, end );

    	if( end + 1 < notes.length() )
    	{
    		while( end + 1 < notes.length() && notes.substring( end + 1 ).matches( "^[\\r\\n\\s]" ) )
    			end++;
    		ret[1] = notes.substring( end + 1 );
    	}
    	
    	Log.d( TAG, "Desc: " + ret[0] );
    	Log.d( TAG, "Note: " + ret[1] );
    	return ret;
    }

    /*******************************************************
     * FORM MODIFIERS
     ******************************************************/
	
	public void resetForm( boolean clearTrip )
	{
		Log.t( TAG, "resetForm( " + clearTrip + " )" );
		if( eventChanged && !isChangeDiscardable() )
		{
			setMessage( TAG, "Event Reset cancelled" );
			return;
		}
		else if( tripChanged && !isChangeDiscardable() )
		{
			setMessage( TAG, "Trip Reset cancelled" );
			return;
		}
		
		eventChanged = false;
		nearbyTab.clear();
    	selectEventType( EventType.EMPTY );

    	if( clearTrip )
		{
			Log.d( TAG,  "Clearing Trip Data..." );
			tripChanged = false;
			selectTrip( TripDbRecord.EMPTY );
		}
		else
		{
			Log.d( TAG, "Resetting form to selected event..." );
			selectEvent( getSelectedEvent() );
		}
	}
	
	public boolean isChangeDiscardable()
	{
		return dialogs.yesNoDialog( "All changes will be lost. Continue?", "WARNING" );
	}
	
	/*******************************************************
     * USER SELECTION ACTION LISTENERS
     ******************************************************/
    
    public void onTripSelect()
    {
		Log.t( TAG, "onTripSelect()" );

		// Only database items will ever be selected by the drop-down
		// It is important to compare the file data with the DB prior 
		// to updating only.
		TripDbRecord t = getSelectedTrip();
		Log.d( TAG, "TripRecord " + t + " selected" );

		if( t == TripDbRecord.NEW )
		{
			ComboDialog<TripDbRecordDisplayItem> cd = new ComboDialog<>( "Trip", cbTrip );
			if( cd.showComboDialog( frame ) )
			{
				String name = cd.getUserText();
				Log.d( TAG, "Creating new trip '" + name + "'!" );
				TripDbRecord newTrip = controller.createTrip( name );
				if( newTrip != TripDbRecord.EMPTY )
				{
					tripChanged = true;
					controller.updateTripList();
					t = newTrip;
				}
			}
			else 
			{
				if( cd.isCancelled() )
					setMessage( TAG, "New trip cancelled!" );
				else
					DmtController.setError( TAG, "Failed to create trip - " + cd.getReason() );
				
				t = TripDbRecord.EMPTY;
			}
		}
		else if( t != TripDbRecord.EMPTY )
		{
			resetForm( true );
			if( !controller.openTrip( t ) )
				return;
		}
		
		selectTrip( t );
		enableOnTripSelected();
    }

	public void onEventSelect()
	{
		Log.t( TAG, "onEventSelect()" );
		
		if( eventChanged && !isChangeDiscardable() )
		{
			Log.d( TAG, "Event change aborted" );
			return;
		}
		
		eventChanged = false;
		EventRecord e = getSelectedEvent();
		Log.d( TAG, "EventRecord " + e + " selected" );

		if( e == EventRecord.NEW )
		{
			ComboDialog<EventRecordDisplayItem> cd = new ComboDialog<>( "Event", cbEvent );
			if( cd.showComboDialog( frame ) )
			{
				String name = cd.getUserText();
				Log.d( TAG, "Creating new event '" + name + "'!" );
				EventRecord newEvent = controller.createEvent( name );
				if( newEvent != EventRecord.EMPTY )
				{
					tripChanged = true;
					controller.updateEventList( e.getTripId() );
					e = newEvent;
				}
			}
			else 
			{
				if( cd.isCancelled() )
					setMessage( TAG, "New event cancelled!" );
				else
					DmtController.setError( TAG, "Failed to create event - " + cd.getReason() );
				e = EventRecord.EMPTY;
			}
		}

		selectEvent( e );
	}

	public void onEventTypeSelect()
	{
		Log.t( TAG, "onEventTypeSelect()" );

		EventType t = getSelectedEventType();
		Log.d( TAG, "EventType " + t + " selected" );

		if( t == EventType.NEW )
		{
			EventRecord er = getSelectedEvent();
			ComboDialog<EventTypeDisplayItem> cd = new ComboDialog<>( "Type", cbEventType );
			if( cd.showComboDialog( frame ) )
			{
				String name = cd.getUserText();
				Log.d( TAG, "Creating new event type '" + name + "'!" );
				EventType newType = controller.createEventType( name );
				if( newType != EventType.EMPTY )
				{
					eventChanged = true;
					controller.updateEventTypeList();
					t = newType;
				}
				else
				{
					t = er.getEventSubType().getType();
				}
			}
			else 
			{
				if( cd.isCancelled() )
					setMessage( TAG, "New event type cancelled!" );
				else
					DmtController.setError( TAG, "Failed to create event type - " + cd.getReason() );

				t = er.getEventSubType().getType();
			}
		}
		
		selectEventType( t );
		enableOnEventTypeSelected();
	}
	
	public void onEventSubTypeSelect()
	{
		Log.t( TAG, "onEventSubTypeSelect()" );

		EventSubType t = getSelectedEventSubType();
		Log.d( TAG, "EventSubType " + t + " selected" );

		if( t == EventSubType.NEW )
		{
			EventType et = getSelectedEventType();
			EventRecord er = getSelectedEvent();
			ComboDialog<EventSubTypeDisplayItem> cd = new ComboDialog<>( "Subtype", cbEventSubType );
			if( cd.showComboDialog( frame ) )
			{
				String name = cd.getUserText();
				Log.d( TAG, "Creating new event subtype '" + name + "'!" );
				EventSubType newType = controller.createEventSubType( et, name );
				if( newType != EventSubType.EMPTY )
				{
					eventChanged = true;
					controller.updateEventSubTypeList( et );
					t = newType;
				}
				else
					t = er.getEventSubType();
			}
			else 
			{
				if( cd.isCancelled() )
					setMessage( TAG, "New event subtype cancelled!" );
				else
					DmtController.setError( TAG, "Failed to create event subtype - " + cd.getReason() );
				t = er.getEventSubType();
			}
			
		}

		selectEventSubType( t );
	}
	
	public boolean isEventDataChanged()
	{
		Log.t( TAG, "isEventDataChanged()" );
		EventRecord er = getEventDataFromForm();
		Log.d( TAG, "Comparing Objects:" );
		Log.d( TAG, "   Original: " + getSelectedEvent() );
		Log.d( TAG, "       Form: " + er );
		return er == getSelectedEvent();
	}
	
	/*******************************************************
     * ACTION HANDLERS
     ******************************************************/

	public void openFile()
	{
		Log.t( TAG, "openFile()" );
		FileSelectionTool fc = new FileSelectionTool();
		fc.setBmkCsvFilter();
		if( fc.display( frame, EventFormMenu.CSV_FILE ) )
		{
			Log.d( TAG, "Opening " + fc.getFile().toString() + "..." );
			TripDbRecord trip = controller.openFile( fc.getFile() );
			if( trip == null )
				return;
			
			selectTrip( trip );
			enableOnOpenTrip();
			setMessage( TAG, "Trip " + trip.getTripName() + " loaded" );
		}
	}

	public void openLegacyFile()
	{
		Log.t( TAG, "openLegacyFile()" );
		FileSelectionTool fc = new FileSelectionTool();
		fc.setCsvFilter();
		if( fc.display( frame, EventFormMenu.LEGACY_FILE ) )
		{
			Log.d( TAG, "Opening " + fc.getFile().toString() + "..." );
			TripDbRecord trip = controller.openFile( fc.getFile() );
			if( trip == null )
				return;
			
			selectTrip( trip );
			enableOnOpenTrip();
			setMessage( TAG, "Trip " + trip.getTripName() + " loaded" );
		}
	}

	public void saveAsFile()
	{
		Log.t( TAG, "saveAsFile()" );
		FileSelectionTool fc = new FileSelectionTool();
		fc.setCsvFilter();
		if( fc.display( frame, EventFormMenu.CSV_FILE ) )
		{
			if( controller.saveAsFile( fc.getFile() ) )
				enableOnSaveAs();
		}
	}
	
	public void saveBookmarkFile()
	{
		Log.t( TAG, "saveookmarkFile()" );
		if( controller.saveFile( false ) )
			enableOnBookmark();
	}

	public void setEndTime()
	{
		Log.t( TAG, "setEndTime()" );

		int idx = cbEvent.getSelectedIndex();
		EventRecordDisplayItem erdi = (EventRecordDisplayItem) cbEvent.getSelectedItem();
		EventRecord er = erdi.getRecord();
		
		Log.d( TAG, "Event: " + er.getDescription() + ", Time: " + 
				LocationRecord.getDateString( er.getMobileData().getDate() ) + 
				", Long: " + er.getMobileData().getDate().getTime() );

		if( idx + 1 > cbEvent.getItemCount() )
		{
			DmtController.setError( TAG, "Cannot assign end time - last record!" );
			return;
		}

		DistanceRecord dr = DistanceRecord.EMPTY;
		EventRecord next = ((EventRecordDisplayItem) cbEvent.getItemAt( idx + 1 )).getRecord();
		
		boolean matches = er.getMobileData().getLocation().equals( next.getMobileData().getLocation() );
		Log.d( TAG, "Current Location: " + er.getMobileData().getLocation().getDisplayString() );
		Log.d( TAG, "   Next Location: " + next.getMobileData().getLocation().getDisplayString() );

		// TODO - Consider adding various options
		try
		{
			dr = DmtController.getGeocoder().getDistance( er, next );
			String status = DmtController.getGeocoder().getLastRequestStatus();
			if( status.equals( Geocoder.STATUS_OK ) )
				erdi.setDistance( dr );
			else
			{
				dr = DistanceRecord.EMPTY;
				DmtController.setError( TAG, "Last request failed, RC=" + status );
			}
		}
		catch( IOException e )
		{
			Log.e( TAG, "Failed to get distance for " + er + " - " + e );
		}
		
		if( dr == DistanceRecord.EMPTY && matches )
		{
			Log.d( TAG, "Locations match, using time - 1" );
			dr = DistanceRecord.EMPTY.copy();
			dr.setOrigin( er.getGeoData() );
			dr.setDestination( next.getGeoData() );
			dr.setDistance( 0 );
			dr.setTextDistance( "same" );
			dr.setDuration( 1 ); // use the next time minus a second
			dr.setTextDuration( "1 sec" );
		}
		else if( dr.getDuration() == 0 )
			dr.setDuration( 1 );

		if( dr == DistanceRecord.EMPTY )
		{
			DmtController.setError( TAG, "Failed to assign end time" );
			return;
		}
			
		long time = next.getMobileData().getDate().getTime() - dr.getDuration() * 1000;
		Log.d( TAG, "Next Time: " + LocationRecord.getDateString( next.getMobileData().getDate() ) + " (" +
				next.getMobileData().getDate().getTime() + "), Dur: " + dr.getDuration() +
				", Result: " + time );
		er.getMobileData().setEndTime( new Date( time ) );
		selectEvent( er );
		enableOnSetEndTime();
		setMessage( TAG, "End time set: " + LocationRecord.getDateString( er.getMobileData().getEndTime() ) );
	}

	public void saveEvent()
	{
		Log.t( TAG, "saveEvent()" );
		// This should both save the file and update DB
		EventRecord event = getEventDataFromForm();
		//TODO change of Date will cause a match to fail
		ArrayList<EventRecord> list = controller.saveEvent( event );
		if( list != null )
		{
			tripChanged = false;
			eventChanged = false;
			
			setEventList( list );
			enableOnSaveEvent();
			setMessage( TAG, "Event[" + event.getEventId() + "] - " + event.getDescription() + " Saved!" );
		}
	}

	public void deleteEvent()
	{
		Log.t( TAG, "deleteEvent()" );
		EventRecord er = getSelectedEvent();
		if( !dialogs.yesNoDialog( "Do you really wish to delete " + er + "?", "CAUTION" ) )
			return;

		if( controller.removeEvent( er ) )
		{
			tripChanged = true;
			controller.updateEventList( er.getTripId() );
			selectEvent( EventRecord.EMPTY );
			enableOnDeleteEvent();
			DmtController.setMessage( TAG, "Event '" + er + "' removed!" );
		}
	}

	public void closeTrip()
	{
		Log.t( TAG, "closeTrip()" );
		String name = getSelectedTrip().getTripName();
		if( ( eventChanged || tripChanged ) && !isChangeDiscardable() )
		{
			setMessage( TAG, "Close cancelled" );
			return;
		}

		ArrayList<TripDbRecord> trips = controller.closeFile();
		if( trips != null )
		{
			resetForm( true );
			setTripList( trips );
			enableOnCloseTrip();
			setMessage( TAG, "Trip " + name + " closed" );
		}
	}
	
	public void geocodeLocation()
	{
		Log.t( TAG, "geocodeLocation()" );
		EventRecord guiData = getEventDataFromForm();
		EventRecord storedData = getSelectedEvent();
		
		if( guiData.getGeoData() != storedData.getGeoData() && !isChangeDiscardable() ) 
		{
			setMessage( TAG, "Geocode cancelled!" );
			return;
		}
		
		ArrayList<MatchRecord> list = controller.geocode( guiData.getGeoData().getFormattedAddress() );
		if( list == null || list.size() == 0 )
			setMessage( TAG, "List empty, geocode failed" );

		else if( list.size() == 1 )
		{
			Log.d( TAG, "List returned a size of 1, exact match" );
			guiData.setGeoData( list.get(0).getRecord() );
			selectEvent( guiData );
			eventChanged = ( guiData.getGeoData() != storedData.getGeoData() );
			enableOnGeocode();
			setMessage( TAG, "Geocode successful!" );
		}
		else
		{
			Log.d( TAG, "List returned " + list.size() + " items, user action required" );
			nearbyTab.setNearbyLocations( list );
			tabbedPane.setSelectedIndex( 1 );
			setMessage( TAG, "Multiple associations exist!" );
		}
	}
	
	public void updateAssociation()
	{
		Log.t( TAG, "updateAssociation()" );
		EventRecord guiData = getEventDataFromForm();
		EventRecord storedData = getSelectedEvent();
		
		if( guiData.getGeoData() != storedData.getGeoData() && !isChangeDiscardable() ) 
		{
			setMessage( TAG, "Update associatiation cancelled!" );
			return;
		}
		
		guiData = controller.updateAssociation( guiData );
		if( guiData != null )
		{
			selectEvent( guiData );
			eventChanged = ( guiData.getGeoData() != storedData.getGeoData() );
			enableOnUpdateAssociation();
			setMessage( TAG, "Association confirmed!" );
		}
		else
		{
			tabbedPane.setSelectedIndex( 1 );
			setMessage( TAG, "Multiple associations exist!" );
		}
	}
	
	public void saveTrip()
	{
		Log.t( TAG, "saveTrip()" );
		TripDbRecord tdr = controller.commitChanges();
		if( tdr != TripDbRecord.EMPTY )
		{
			setEventList( tdr.getList() );
			enableOnSaveTrip();
			setMessage( TAG, "Trip[" + tdr.getTripId() + "] " + tdr.getTripName() + " saved!" );
		}
	}
	
	public void changeTrip()
	{
		Log.t( TAG, "changeTrip()" );
		//TODO - build dialog with list of trips,ok/cancel
		//	onOk() - change trip_id in record, update drop-down
		// This change is automatic
		eventChanged = false;
		enableOnChangeTrip();
		setMessage( TAG, "Event changed to new trip" );
	}
	
	public void setAssociation()
	{
		Log.t( TAG, "setAssociation()" );
		MatchRecord mr = nearbyTab.getSelectedObject();
		EventRecord er = controller.setAssociation( mr );
		if( er != null )
		{
			er.getGeoData().setGeoLocation( mr.getRecord().getGeoLocation() );
			selectEvent( er );
			eventChanged = true;	// Must come after setEvent to ensure change marked
			tabbedPane.setSelectedIndex( 0 );
			enableOnSetAssociation();
			setMessage( TAG, "Assigned new association to event" );
		}
	}
	
	public void dbConnect( boolean connect )
	{
		Log.t( TAG, "dbConnect( " + connect + " )" );
		controller.connectToDb( connect );
		if( connect )
    		enableOnDbConnect();

		else
    		enableOnDbDisconnect();
	}
	
	public void importDb()
	{
		Log.t( TAG, "importDb()" );
		if( DmtController.getInstance().yesNoDialog( "Do you wish to remove the existing database?", "Import Database" ) )
		{
			FileSelectionTool fc = new FileSelectionTool();
			fc.setSqlFilter();
			if( fc.display( DmtController.getDisplay().getFrame(), "SQL File" ) )
			{
				if( DmtController.getInstance().importDb( fc.getFile() ) )
					enableOnImportDb();
			}
		}
		else
			DmtController.setMessage( TAG, "Import Database Cancelled!" );
	}
	
	public void exportDb()
	{
		Log.t( TAG, "exportDb()" );
		FileSelectionTool fc = new FileSelectionTool();
		fc.setSqlFilter();
		if( fc.display( DmtController.getDisplay().getFrame(), "SQL File" ) )
		{
			if( DmtController.getInstance().exportDb( fc.getFile() ) )
				enableOnExportDb();
		}
	}

	/**
	 * References:
	 *    https://docs.qgis.org/2.2/en/docs/user_manual/introduction/getting_started.html
	 */
	public void startQgis()
	{
		Log.t( TAG, "startQgis()" );
		new ExternalProgram( QGIS_PROGRAM, this ).start();
		enableOnQgisOpen();
	}

	public void startAnalyzer()
	{
		Log.t( TAG, "startAnalyzer()" );
		AnalysisForm form = new AnalysisForm();
		enableOnAnalyzeOpen();
		form.display();
	}

	@Override
	public void programStopped( String cmd )
	{
		// This will be called on close
		Log.t( TAG, "programStopped()" );
		if( cmd.equals( QGIS_PROGRAM ) )
			enableOnQgisClose();
		else if( cmd.equals( ANALYZE_PROGRAM ) )
			enableOnAnalyzeClose();
	}
	
	public void closeProgram()
	{
		Log.t( TAG, "closeProgram()" );
		if( !( tripChanged || eventChanged ) || isChangeDiscardable() )
			controller.exitProgram();

		eventChanged = false;
		DmtController.setMessage( TAG, "Close Cancelled!" );
	}
	
	public void searchNearby()
	{
		Log.t( TAG, "searchNearby()" );
		EventRecord er = getSelectedEvent();

		ArrayList<MatchRecord> list = controller.searchNearby( er.getMobileData().getLocation(),
				nearbyTab.getRadius(), nearbyTab.getKeyword() );

		nearbyTab.clear();
		if( list == null || list.size() == 0 )
			setMessage( TAG, "List empty, geocode failed" );

		else if( list.size() > 0 )
		{
			Log.d( TAG, "List returned " + list.size() + " items, user action required" );
			nearbyTab.setNearbyLocations( list );
			enableOnSearch();
			setMessage( TAG, "Search processing complete!" );
		}
	}

	/*******************************************************
     * FORM FEATURE ENABLERS
     ******************************************************/
	
	@Override
	public void enableOnFormCreate() 
	{
		Log.t( TAG, "enableOnFormCreate()" );
		eventMenu.enableOnFormCreate();
		nearbyTab.enableOnFormCreate();
		locationTab.enableOnFormCreate();
		btnSave.setEnabled( false );
		btnReset.setEnabled( false );
		btnDelete.setEnabled( false );
		btnClose.setEnabled( true );
		btnClose.setText( EventForm.EXIT );

		cbTrip.setEnabled( true );
		cbEvent.setEnabled( false );
		txtDate.setEnabled( false );
		cbEventType.setEnabled( false );
		cbEventSubType.setEnabled( false );
		txtEndTime.setEnabled( false );
		txtNotes.setEnabled( false );
	}

	@Override
	public void enableOnChanged() 
	{
		Log.t( TAG, "enableOnChanged()" );
		eventMenu.enableOnChanged();
		nearbyTab.enableOnChanged();
		locationTab.enableOnChanged();
		btnSave.setEnabled( true );
		btnReset.setEnabled( true );
	}

	@Override
	public void enableOnDbConnect() 
	{
		Log.t( TAG, "enableOnDbConnect()" );
		eventMenu.enableOnDbConnect();
		nearbyTab.enableOnDbConnect();
		locationTab.enableOnDbConnect();
	}

	@Override
	public void enableOnDbDisconnect() 
	{
		Log.t( TAG, "enableOnDbDisconnect()" );
		eventMenu.enableOnDbDisconnect();
		nearbyTab.enableOnDbDisconnect();
		locationTab.enableOnDbDisconnect();
	}

	@Override
	public void enableOnOpenTrip() 
	{
		Log.t( TAG, "enableOnOpenTrip()" );
		eventMenu.enableOnOpenTrip();
		nearbyTab.enableOnOpenTrip();
		locationTab.enableOnOpenTrip();
		cbTrip.setEnabled( false );
		cbEvent.setEnabled( true );
		btnClose.setText( EventForm.CLOSE );
	}

	@Override
	public void enableOnSaveAs() 
	{
		Log.t( TAG, "enableOnSaveAs()" );
		eventMenu.enableOnSaveAs();
		nearbyTab.enableOnSaveAs();
		locationTab.enableOnSaveAs();
	}

	@Override
	public void enableOnBookmark() 
	{
		Log.t( TAG, "enableOnBookmark()" );
		eventMenu.enableOnBookmark();
		nearbyTab.enableOnBookmark();
		locationTab.enableOnBookmark();
	}


	@Override
	public void enableOnSetEndTime() 
	{
		Log.t( TAG, "enableOnSetEndTime()" );
		eventMenu.enableOnSetEndTime();
		nearbyTab.enableOnSetEndTime();
		locationTab.enableOnSetEndTime();
	}
	
	@Override
	public void enableOnSaveTrip()
	{
		Log.t( TAG, "enableOnSaveTrip()" );
		eventMenu.enableOnSaveTrip();
		nearbyTab.enableOnSaveTrip();
		locationTab.enableOnSaveTrip();
	}
	
	@Override
	public void enableOnImportDb()
	{
		Log.t( TAG, "enableOnImportDb()" );
		eventMenu.enableOnImportDb();
		nearbyTab.enableOnImportDb();
		locationTab.enableOnImportDb();
		//No need to do this - miDbImport.setEnabled( false );
	}
	
	@Override
	public void enableOnExportDb()
	{
		Log.t( TAG, "enableOnExportDb()" );
		eventMenu.enableOnExportDb();
		nearbyTab.enableOnExportDb();
		locationTab.enableOnExportDb();
	}

	@Override
	public void enableOnQgisOpen()
	{
		Log.t( TAG, "enableOnQgisOpen()" );
		eventMenu.enableOnQgisOpen();
		nearbyTab.enableOnQgisOpen();
		locationTab.enableOnQgisOpen();
	}
	
	@Override
	public void enableOnQgisClose()
	{
		Log.t( TAG, "enableOnQgisClose()" );
		eventMenu.enableOnQgisClose();
		nearbyTab.enableOnQgisClose();
		locationTab.enableOnQgisClose();
	}
	
	@Override
	public void enableOnAnalyzeOpen()
	{
		Log.t( TAG, "enableOnAnalyzeOpen()" );
		eventMenu.enableOnAnalyzeOpen();
		nearbyTab.enableOnAnalyzeOpen();
		locationTab.enableOnAnalyzeOpen();
	}
	
	@Override
	public void enableOnAnalyzeClose()
	{
		Log.t( TAG, "enableOnAnalyzeClose()" );
		eventMenu.enableOnAnalyzeClose();
		nearbyTab.enableOnAnalyzeClose();
		locationTab.enableOnAnalyzeClose();
	}

	@Override
	public void enableOnResetEvent() 
	{
		Log.t( TAG, "enableOnResetEvent()" );
		eventMenu.enableOnResetEvent();
		nearbyTab.enableOnResetEvent();
		locationTab.enableOnResetEvent();
		//TODO - if !eventChanged::TRUE btnSave.setEnabled( false );
		//TODO - if !eventChanged::TRUE btnReset.setEnabled( false );
	}

	@Override
	public void enableOnDeleteEvent() 
	{
		Log.t( TAG, "enableOnDeleteEvent()" );
		eventMenu.enableOnDeleteEvent();
		nearbyTab.enableOnDeleteEvent();
		locationTab.enableOnDeleteEvent();
	}

	@Override
	public void enableOnSaveEvent() 
	{
		Log.t( TAG, "enableOnSaveEvent()" );
		eventMenu.enableOnSaveEvent();
		nearbyTab.enableOnSaveEvent();
		locationTab.enableOnSaveEvent();
		//TODO - if !eventChanged::TRUE btnSave.setEnabled( false );
		//TODO - if !eventChanged::TRUE btnReset.setEnabled( false );
	}

	@Override
	public void enableOnCloseTrip() 
	{
		Log.t( TAG, "enableOnCloseTrip()" );
		eventMenu.enableOnCloseTrip();
		nearbyTab.enableOnCloseTrip();
		locationTab.enableOnCloseTrip();
		btnSave.setEnabled( false );
		btnReset.setEnabled( false );
		btnDelete.setEnabled( false );
		btnClose.setText( EventForm.EXIT );

		cbTrip.setEnabled( true );
		cbEvent.setEnabled( false );
		txtDate.setEnabled( false );
		cbEventType.setEnabled( false );
		cbEventSubType.setEnabled( false );
		txtEndTime.setEnabled( false );
		txtNotes.setEnabled( false );
	}

	@Override
	public void enableOnTripSelected() 
	{
		Log.t( TAG, "enableOnTripSelected()" );
		eventMenu.enableOnTripSelected();
		nearbyTab.enableOnTripSelected();
		locationTab.enableOnTripSelected();
		btnClose.setText( EventForm.CLOSE );
		cbTrip.setEnabled( false );
		cbEvent.setEnabled( true );
		cbEventType.setEnabled( false );
		cbEventSubType.setEnabled( false );
	}

	@Override
	public void enableOnEventSelected( boolean editableEvent ) 
	{
		Log.t( TAG, "enableOnEventSelected( " + editableEvent + " )" );
		eventMenu.enableOnEventSelected( editableEvent );
		nearbyTab.enableOnEventSelected( editableEvent );
		locationTab.enableOnEventSelected( editableEvent );
		btnSave.setEnabled( true ); // TODO - use enableOnChanged
		btnReset.setEnabled( true ); // TODO - use enableOnChanged
		btnDelete.setEnabled( editableEvent );

		boolean editableSubType = ( getSelectedEvent().getEventSubType().getType() != EventType.EMPTY );

		txtDate.setEnabled( editableEvent );
		cbEventType.setEnabled( editableEvent );
		cbEventSubType.setEnabled( editableSubType );
		txtEndTime.setEnabled( editableEvent );
		txtNotes.setEnabled( editableEvent );
	}

	@Override
	public void enableOnEventTypeSelected() 
	{
		Log.t( TAG, "enableOnEventTypeSelected()" );
		eventMenu.enableOnEventTypeSelected();
		nearbyTab.enableOnEventTypeSelected();
		locationTab.enableOnEventTypeSelected();

		boolean editableEvent = ( getSelectedEvent() != EventRecord.EMPTY );
		cbEventSubType.setEnabled( editableEvent );
	}

	@Override
	public void enableOnEventSubTypeSelected() 
	{
		Log.t( TAG, "enableOnEventSubTypeSelected()" );
		eventMenu.enableOnEventSubTypeSelected();
		nearbyTab.enableOnEventSubTypeSelected();
		locationTab.enableOnEventSubTypeSelected();
	}

	@Override
	public void enableOnSetAssociation() 
	{
		Log.t( TAG, "enableOnSetAssociation()" );
		eventMenu.enableOnSetAssociation();
		nearbyTab.enableOnSetAssociation();
		locationTab.enableOnSetAssociation();
	}

	@Override
	public void enableOnSearch() 
	{
		Log.t( TAG, "enableOnSearch()" );
		eventMenu.enableOnSearch();
		nearbyTab.enableOnSearch();
		locationTab.enableOnSearch();
		tabbedPane.setSelectedIndex( 1 );
	}

	@Override
	public void enableOnRefresh() 
	{
		Log.t( TAG, "enableOnRefresh()" );
		eventMenu.enableOnRefresh();
		nearbyTab.enableOnRefresh();
		locationTab.enableOnRefresh();
	}

	@Override
	public void enableOnMatchSelected() 
	{
		Log.t( TAG, "enableOnMatchSelected()" );
		eventMenu.enableOnMatchSelected();
		nearbyTab.enableOnMatchSelected();
		locationTab.enableOnMatchSelected();
	}

	@Override
	public void enableOnGeocode() 
	{
		Log.t( TAG, "enableOnGeocode()" );
		eventMenu.enableOnGeocode();
		nearbyTab.enableOnGeocode();
		locationTab.enableOnGeocode();
	}

	@Override
	public void enableOnChangeTrip() 
	{
		Log.t( TAG, "enableOnChangeTrip()" );
		eventMenu.enableOnChangeTrip();
		nearbyTab.enableOnChangeTrip();
		locationTab.enableOnChangeTrip();
	}

	@Override
	public void enableOnUpdateAssociation() 
	{
		Log.t( TAG, "enableOnUpdateAssociation()" );
		eventMenu.enableOnUpdateAssociation();
		nearbyTab.enableOnUpdateAssociation();
		locationTab.enableOnUpdateAssociation();
	}
}
