package com.tripgis.dmt.ui;

import java.awt.FlowLayout;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.types.EventRecord;
import com.tripgis.dmt.types.GeocodeRecord;
import com.tripgis.dmt.ui.listeners.EventButtonActionListener;
import com.tripgis.dmt.ui.listeners.EventEnabler;
import com.tripgis.dmt.util.LatLng;
import com.tripgis.dmt.util.LogTranslator;

public class EventFormLocationTab implements EventEnabler
{
    // Name to use in log lines
    public static final String TAG = EventFormLocationTab.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

	private JPanel container;
	
	JTextField txtLatLng;
	JTextField txtStore;
	JTextField txtAddress;
	JTextField txtCity;
	JTextField txtState;
	JTextField txtZip;
	JTextField txtCountry;
	JTextField txtCounty;
	JTextField txtUtcOffset;
	JTextField txtPlaceId;
	JTextField txtGeocodeLatLng;
	JTextField txtLocationType;

	JButton btnGeocode;
	JButton btnUpdateAssoc;
	JButton btnChangeTrip;
	
	public EventFormLocationTab()
	{
		Log.t( TAG, "EventFormLocationTab()" );
	}

	public void initialize()
	{
		Log.t( TAG, "initialize()" );
		initializeTab();
	}
	
	private void initializeTab()
	{
		Log.t( TAG, "initializeTab()" );
		container = new JPanel();
		
		JPanel locationDetails = new JPanel();
		JPanel buttons1 = new JPanel();
		JPanel buttons2 = new JPanel();

		txtLatLng        = new JTextField( 20 );
		txtStore         = new JTextField( 20 );
		txtAddress       = new JTextField( 20 );
		txtCity          = new JTextField( 20 );
		txtState         = new JTextField( 20 );
		txtZip           = new JTextField( 20 );
		txtCountry       = new JTextField( 20 );
		txtCounty        = new JTextField( 20 );
		txtUtcOffset     = new JTextField( 20 );
		txtPlaceId       = new JTextField( 20 );
		txtLocationType  = new JTextField( 20 );
		txtGeocodeLatLng = new JTextField( 20 );
		
		txtLatLng.setEditable( false );
		txtPlaceId.setEditable( false );
		txtLocationType.setEditable( false );
		txtGeocodeLatLng.setEditable( false );
		
		JLabel latlng  = new JLabel( "Lat/Long" );
		JLabel store   = new JLabel( "Store" );
		JLabel address = new JLabel( "Address" );
		JLabel city    = new JLabel( "City" );
		JLabel state   = new JLabel( "State" );
		JLabel zip     = new JLabel( "Zip" );
		JLabel country = new JLabel( "Country" );
		JLabel county  = new JLabel( "County" );
		JLabel utc     = new JLabel( "Offset" );
		JLabel placeid = new JLabel( "PlaceId" );
		JLabel loctype = new JLabel( "LocType" );
		JLabel geoll   = new JLabel( "GeoPos" );

		JLabel[] labels     = new JLabel[12];
		labels[0]  = latlng;
		labels[1]  = store;
		labels[2]  = address;
		labels[3]  = city;
		labels[4]  = state;
		labels[5]  = zip;
		labels[6]  = country;
		labels[7]  = county;
		labels[8]  = utc;
		labels[9]  = placeid;
		labels[10] = loctype;
		labels[11] = geoll;
		
		JComponent[] fields = new JComponent[12];
		fields[0]  = txtLatLng;
		fields[1]  = txtStore;
		fields[2]  = txtAddress;
		fields[3]  = txtCity;
		fields[4]  = txtState;
		fields[5]  = txtZip;
		fields[6]  = txtCountry;
		fields[7]  = txtCounty;
		fields[8]  = txtUtcOffset;
		fields[9]  = txtPlaceId;
		fields[10]  = txtLocationType;
		fields[11] = txtGeocodeLatLng;
		
		GroupLayout layout = EventForm.prepareGroupLayout( locationDetails, labels, fields );		
		locationDetails.setLayout( layout );
		layout.setAutoCreateGaps( true );

		btnGeocode = new JButton( EventForm.GEOCODE );
		btnUpdateAssoc = new JButton( EventForm.UPDATE_ASSOC );
		btnChangeTrip  = new JButton( EventForm.CHANGE_TRIP );
		
		EventButtonActionListener listener = new EventButtonActionListener();
		btnGeocode.addActionListener( listener );
		btnUpdateAssoc.addActionListener( listener );
		btnChangeTrip.addActionListener( listener );

		buttons1.setLayout( new FlowLayout() );
		buttons1.add( btnGeocode );
		buttons1.add( btnUpdateAssoc );
		buttons1.add( btnChangeTrip );
		
		container.add( locationDetails );
		container.add( buttons1 );

		buttons2.setLayout( new FlowLayout() );

		container.add( buttons2 );
		
		SpringLayout eLayout = new SpringLayout();
		container.setLayout( eLayout );
		eLayout.putConstraint( SpringLayout.WEST, locationDetails, 5, SpringLayout.WEST, container );
		eLayout.putConstraint( SpringLayout.HORIZONTAL_CENTER, buttons1, 0, SpringLayout.HORIZONTAL_CENTER, container );
		eLayout.putConstraint( SpringLayout.HORIZONTAL_CENTER, buttons2, 0, SpringLayout.HORIZONTAL_CENTER, container );
		eLayout.putConstraint( SpringLayout.NORTH, locationDetails, 5, SpringLayout.NORTH, container );
		eLayout.putConstraint( SpringLayout.NORTH, buttons1, 5, SpringLayout.SOUTH, locationDetails );
		eLayout.putConstraint( SpringLayout.NORTH, buttons2, 5, SpringLayout.SOUTH, buttons1 );
//		eLayout.putConstraint( SpringLayout.EAST, container, 5, SpringLayout.EAST, buttons );
		eLayout.putConstraint( SpringLayout.EAST, container, 5, SpringLayout.EAST, locationDetails );
		eLayout.putConstraint( SpringLayout.SOUTH, container, 5, SpringLayout.SOUTH, buttons2 );
	}

	public JPanel getTab() { return container; }
	
	public void setEvent( EventRecord e )
	{
		Log.t( TAG, "setEvent( " + e + " )" );
		LatLng latlng = e.getMobileData().getLocation();
		if( latlng != null && latlng.isValid() )
			txtLatLng.setText( latlng.getDisplayString() );
		else
			txtLatLng.setText( "" );

		txtStore.setText( e.getGeoData().getStore() );
		txtAddress.setText( e.getGeoData().getAddress() );
		txtCity.setText( e.getGeoData().getCity() );
		txtState.setText( e.getGeoData().getState() );
		txtZip.setText( e.getGeoData().getZip() );
		txtCountry.setText( e.getGeoData().getCountry() );
		txtCounty.setText( e.getGeoData().getCounty() );
		txtUtcOffset.setText( e.getGeoData().getUtcOffset() );
		txtLocationType.setText( e.getGeoData().getLocationType() );
		txtPlaceId.setText( e.getGeoData().getPlaceId() );
		txtGeocodeLatLng.setText( e.getGeoData().getGeoLocation().getDisplayString() );
	}
	
	public void getEventDataFromTab( EventRecord er )
	{
		Log.t( TAG, "getEventDataFromTab( " + er + " )" );
		GeocodeRecord gr = GeocodeRecord.EMPTY.copy();
		gr.setAddress( txtAddress.getText() );
		gr.setStore( txtStore.getText() );
		gr.setCity( txtCity.getText() );
		gr.setState( txtState.getText() );
		gr.setZip( txtZip.getText() );
		gr.setCountry( txtCountry.getText() );
		gr.setCounty( txtCounty.getText() );
		gr.setUtcOffset( txtUtcOffset.getText() );
		gr.setPlaceId( txtPlaceId.getText() );
		gr.setLocationType( txtLocationType.getText() );
		gr.setGeoLocation( new LatLng( txtGeocodeLatLng.getText() ) );
		if( er.getMobileData().getLocation() == LatLng.EMPTY )
		{
			Log.d( TAG, "Assigning position from geocode..." );
			er.getMobileData().setLocation( gr.getGeoLocation().getDisplayString() );
		}
		
		Log.d( TAG, "S: " + er.getMobileData().getLocation() + ", E: " + LatLng.EMPTY +
				", C: " + ( er.getMobileData().getLocation() == LatLng.EMPTY ) );
		
		er.setGeoData( gr );
	}


	/*******************************************************
     * FORM FEATURE ENABLERS
     ******************************************************/

	@Override
	public void enableOnFormCreate() 
	{
		//btnChangeAssoc now btnGeocode
		btnGeocode.setEnabled( false );
		btnChangeTrip.setEnabled( false );
		btnUpdateAssoc.setEnabled( false );

		txtStore.setEnabled( false );
		txtAddress.setEnabled( false );
		txtCity.setEnabled( false );
		txtState.setEnabled( false );
		txtZip.setEnabled( false );
		txtCountry.setEnabled( false );
		txtCounty.setEnabled( false );
		txtUtcOffset.setEnabled( false );
		txtPlaceId.setEnabled( false );
		txtLocationType.setEnabled( false );
		txtGeocodeLatLng.setEnabled( false );
	}

	@Override
	public void enableOnChanged() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDbConnect() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDbDisconnect() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnOpenTrip() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveAs() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnBookmark() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSetEndTime() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void enableOnSaveTrip() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void enableOnImportDb() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void enableOnExportDb(){
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnQgisOpen() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnQgisClose() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnAnalyzeOpen() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnAnalyzeClose() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnResetEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDeleteEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnCloseTrip() 
	{
		btnGeocode.setEnabled( false );
		btnChangeTrip.setEnabled( false );
		btnUpdateAssoc.setEnabled( false );

		txtStore.setEnabled( false );
		txtAddress.setEnabled( false );
		txtCity.setEnabled( false );
		txtState.setEnabled( false );
		txtZip.setEnabled( false );
		txtCountry.setEnabled( false );
		txtCounty.setEnabled( false );
	}

	@Override
	public void enableOnTripSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnEventSelected( boolean flag ) 
	{
		btnGeocode.setEnabled( flag );
		btnChangeTrip.setEnabled( flag );
		if( flag && !DmtController.getDisplay().getSelectedEvent().getGeoData().getPlaceId().isEmpty() )
			btnUpdateAssoc.setEnabled( true );
		else
			btnUpdateAssoc.setEnabled( false );

		txtStore.setEnabled( flag );
		txtAddress.setEnabled( flag );
		txtCity.setEnabled( flag );
		txtState.setEnabled( flag );
		txtZip.setEnabled( flag );
		txtCountry.setEnabled( flag );
		txtCounty.setEnabled( flag );
	}

	@Override
	public void enableOnEventTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnEventSubTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSetAssociation() 
	{
		btnUpdateAssoc.setEnabled( !DmtController.getDisplay().getSelectedEvent().getGeoData().getPlaceId().isEmpty() );
	}

	@Override
	public void enableOnSearch() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnRefresh() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnMatchSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnGeocode() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnChangeTrip() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnUpdateAssociation() {
		// TODO Auto-generated method stub
		
	}
}
