package com.tripgis.dmt.ui;

import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import com.tripgis.dmt.ui.listeners.EventEnabler;
import com.tripgis.dmt.ui.listeners.EventMenuActionListener;
import com.tripgis.dmt.util.LogTranslator;

public class EventFormMenu extends JMenuBar implements EventEnabler
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -8969626602964137020L;

	private JMenu menuFileOpen;
	
	private JMenuItem miFileOpenCsv;
	private JMenuItem miFileOpenLegacy;
	private JMenuItem miFileClose;
	private JMenuItem miFileSave;
	private JMenuItem miFileSaveAs;
	private JMenuItem miBookmark;
	private JMenuItem miSetEndTime;
	private JMenuItem miExit;
	
	private JMenuItem miDbConnect;
	private JMenuItem miDbUpdate;
	private JMenuItem miDbExport;
	private JMenuItem miDbImport;
	private JMenuItem miDbExportTrip;
	
	private JMenuItem miGisView;
	private JMenuItem miGisAnalyze;

    // Name to use in log lines
    public static final String TAG = EventFormMenu.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );

	public static final String CSV_FILE   = "CSV File";
	public static final String LEGACY_FILE = "Legacy CSV";
    public static final String CLOSE      = "Close";
    public static final String SAVE       = "Save";
    public static final String SAVE_AS    = "Save As...";
	public static final String BOOKMARK   = "Bookmark";
	public static final String SET_END_TIME = "Set End Time";
	public static final String EXIT       = "Exit";
	public static final String CONNECT    = "Connect";
	public static final String DISCONNECT = "Disconnect";
    public static final String COMMIT_DB  = "Commit to Db";
	public static final String IMPORT_DB  = "Import Db";
	public static final String EXPORT_DB  = "Export Db";
	public static final String EXPORT_TRIP = "Export Trip";
    public static final String GIS_VIEW   = "View";
    public static final String ANALYZE    = "Analyze";
	
	
	public EventFormMenu()
	{
	}

	public void initialize()
	{
		Log.t( TAG, "initialize()" );
		initializeMenu();
	}
	
	/**
	 * References:
	 *    http://docs.oracle.com/javase/tutorial/uiswing/components/button.html#checkbox
	 *    http://www.java2s.com/Tutorial/Java/0240__Swing/CreatingJMenuItemComponents.htm
	 *    http://docs.oracle.com/javase/tutorial/uiswing/components/menu.html
	 *    http://www.java2s.com/Tutorial/Java/0240__Swing/ListeningtoJMenuItemEventswithanActionListener.htm
	 */
	private void initializeMenu()
	{
		//Icon atIcon = new ImageIcon("at.gif");
		//JMenuItem jMenuItem = new JMenuItem("Options", atIcon);
		
		//Action action = ...;
		//JMenuItem jMenuItem = new JMenuItem(action);
		
		KeyStroke ctrlOKeyStroke = KeyStroke.getKeyStroke("control O");
		KeyStroke ctrlXKeyStroke = KeyStroke.getKeyStroke("control X");
		KeyStroke ctrlEKeyStroke = KeyStroke.getKeyStroke("control E");
		
		//elementMenu.add(circleItem = new JRadioButtonMenuItem("Circle", false));
		//circleItem.addActionListener
	    //ButtonGroup types = new ButtonGroup();
	    //types.add(lineItem);
	    
		EventMenuActionListener listener = new EventMenuActionListener();
	    JMenu fileMenu = new JMenu( "File" );
		fileMenu.setMnemonic( KeyEvent.VK_F );
		
		menuFileOpen = new JMenu( "Open..." );
		menuFileOpen.setMnemonic( KeyEvent.VK_O );
		fileMenu.add( menuFileOpen );

		miFileOpenCsv = new JMenuItem( CSV_FILE );
		miFileOpenCsv.setMnemonic( KeyEvent.VK_V );
		miFileOpenCsv.setAccelerator( ctrlOKeyStroke );
		miFileOpenCsv.addActionListener( listener );
		menuFileOpen.add( miFileOpenCsv );
		
		miFileOpenLegacy = new JMenuItem( LEGACY_FILE );
		miFileOpenLegacy.addActionListener( listener );
		menuFileOpen.add( miFileOpenLegacy );
		
		miFileClose = new JMenuItem( CLOSE );
		miFileClose.setMnemonic( KeyEvent.VK_C );
		miFileClose.addActionListener( listener );
		fileMenu.add( miFileClose );
		
		fileMenu.addSeparator();
		miFileSave = new JMenuItem( SAVE );
		miFileSave.setMnemonic( KeyEvent.VK_S );
		miFileSave.addActionListener( listener );
		fileMenu.add( miFileSave );
		
		miFileSaveAs = new JMenuItem( SAVE_AS );
		miFileSaveAs.setMnemonic( KeyEvent.VK_A );
		miFileSaveAs.addActionListener( listener );
		fileMenu.add( miFileSaveAs );

		miBookmark = new JMenuItem( BOOKMARK );
		miBookmark.setMnemonic( KeyEvent.VK_B );
		miBookmark.addActionListener( listener );
		fileMenu.add( miBookmark );

		fileMenu.addSeparator();
		miSetEndTime = new JMenuItem( SET_END_TIME );
		miSetEndTime.setMnemonic( KeyEvent.VK_T );
		miSetEndTime.setAccelerator( ctrlEKeyStroke );
		miSetEndTime.addActionListener( listener );
		fileMenu.add( miSetEndTime );
		
		fileMenu.addSeparator();
		miExit = new JMenuItem( EXIT, KeyEvent.VK_X );
		miExit.setAccelerator( ctrlXKeyStroke );
		miExit.addActionListener( listener );
		fileMenu.add( miExit );
		add( fileMenu );

		JMenu dbMenu = new JMenu( "Db" );
		dbMenu.setMnemonic( KeyEvent.VK_D );
		miDbConnect = new JMenuItem( "Connect" );
		miDbConnect.addActionListener( listener );
		dbMenu.add( miDbConnect );

		miDbUpdate = new JMenuItem( COMMIT_DB, KeyEvent.VK_U );
		miDbUpdate.addActionListener( listener );
		dbMenu.add( miDbUpdate );

		miDbExportTrip = new JMenuItem( EXPORT_TRIP, KeyEvent.VK_U );
		miDbExportTrip.addActionListener( listener );
		dbMenu.add( miDbExportTrip );
		dbMenu.addSeparator();

		miDbImport = new JMenuItem( IMPORT_DB );
		miDbImport.setMnemonic( KeyEvent.VK_I );
		miDbImport.addActionListener( listener );
		dbMenu.add( miDbImport );
		
		miDbExport = new JMenuItem( EXPORT_DB );
		miDbExport.setMnemonic( KeyEvent.VK_E );
		miDbExport.addActionListener( listener );
		dbMenu.add( miDbExport );
		add( dbMenu );

		JMenu gisMenu = new JMenu( "GIS" );
		miGisView = new JMenuItem( GIS_VIEW );
		miGisView.setMnemonic( KeyEvent.VK_G );
		miGisView.addActionListener( listener );
		gisMenu.add( miGisView );
		
		miGisAnalyze = new JMenuItem( ANALYZE );
		miGisAnalyze.setMnemonic( KeyEvent.VK_Z );
		miGisAnalyze.addActionListener( listener );
		gisMenu.add( miGisAnalyze );
		add( gisMenu );
	}
	
	/*******************************************************
     * FORM FEATURE ENABLERS
     ******************************************************/
	
	@Override
	public void enableOnFormCreate() 
	{
		miFileOpenCsv.setEnabled( true );
		miFileOpenLegacy.setEnabled( true );
		miFileSave.setEnabled( false );
		miFileSaveAs.setEnabled( false );
		miBookmark.setEnabled( false ); // TODO
		miFileClose.setEnabled( false );
		miSetEndTime.setEnabled( false );
		miDbConnect.setEnabled( true );
		miDbUpdate.setEnabled( false ); // TODO
		miGisView.setEnabled( false );
		miGisAnalyze.setEnabled( false );
		miDbExportTrip.setEnabled( false );

		if( miDbConnect.getText().equals( DISCONNECT ) )
			miDbImport.setEnabled( true );
	}

	@Override
	public void enableOnChanged() 
	{
		miBookmark.setEnabled( true );
		miFileSave.setEnabled( true );
	}

	@Override
	public void enableOnDbConnect() 
	{
		miDbConnect.setText( DISCONNECT );
		miDbExport.setEnabled( true );
	}

	@Override
	public void enableOnDbDisconnect() 
	{
		miDbConnect.setText( CONNECT );
		miDbExport.setEnabled( false );
		miDbImport.setEnabled( false );
		miDbExportTrip.setEnabled( false );
	}

	@Override
	public void enableOnOpenTrip() 
	{
		miFileOpenCsv.setEnabled( false );
		miFileOpenLegacy.setEnabled( false );
		miFileClose.setEnabled( true );
		miFileSaveAs.setEnabled( true );
		miBookmark.setEnabled( true );

		if( miDbConnect.getText().equals( DISCONNECT ) )
			miDbUpdate.setEnabled( true );

		miGisView.setEnabled( true );
		miGisAnalyze.setEnabled( true );
	}

	@Override
	public void enableOnSaveAs() { }

	@Override
	public void enableOnBookmark() 
	{
		miBookmark.setEnabled( false );
	}
	
	@Override
	public void enableOnSetEndTime() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveTrip()
	{
		miDbUpdate.setEnabled( false );
	}
	
	@Override
	public void enableOnImportDb()
	{
		//No need to do this - miDbImport.setEnabled( false );
	}
	
	@Override
	public void enableOnExportDb()
	{
		miDbExport.setEnabled( false );
	}

	@Override
	public void enableOnQgisOpen()
	{
		miGisView.setEnabled( false );
	}
	
	@Override
	public void enableOnQgisClose()
	{
		miGisView.setEnabled( true );
	}
	
	@Override
	public void enableOnAnalyzeOpen()
	{
		miGisAnalyze.setEnabled( false );
	}
	
	@Override
	public void enableOnAnalyzeClose()
	{
		miGisAnalyze.setEnabled( true );
	}

	@Override
	public void enableOnResetEvent() 
	{
		//TODO - if !eventChanged::TRUE miFileSave.setEnabled( false );
	}

	@Override
	public void enableOnDeleteEvent() {}

	@Override
	public void enableOnSaveEvent() 
	{
		//TODO - if !eventChanged::TRUE miFileSave.setEnabled( false );
	}

	@Override
	public void enableOnCloseTrip() 
	{
		miFileOpenCsv.setEnabled( true );
		miFileOpenLegacy.setEnabled( true );
		miFileSave.setEnabled( false );
		miFileSaveAs.setEnabled( false );
		miBookmark.setEnabled( false );
		miFileClose.setEnabled( false );
		miSetEndTime.setEnabled( false );
		miDbUpdate.setEnabled( false );
		miDbExportTrip.setEnabled( false );
		miGisView.setEnabled( false );
		miGisAnalyze.setEnabled( false );

		if( miDbConnect.getText().equals( DISCONNECT ) )
			miDbImport.setEnabled( true );
	}

	@Override
	public void enableOnTripSelected() 
	{
		miFileOpenCsv.setEnabled( false );
		miFileOpenLegacy.setEnabled( false );
		miFileClose.setEnabled( true );
		//miFileSaveAs.setEnabled( true );
		//miBookmark.setEnabled( true );
		//miDbUpdate.setEnabled( true );
		miDbImport.setEnabled( false );
		miDbExportTrip.setEnabled( true );
		miGisView.setEnabled( true );
		miGisAnalyze.setEnabled( true );
	}

	@Override
	public void enableOnEventSelected( boolean flag ) 
	{
		miFileSave.setEnabled( true ); // TODO - use enableOnChanged
		miSetEndTime.setEnabled( flag );
	}

	@Override
	public void enableOnEventTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnEventSubTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSetAssociation() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSearch() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnRefresh() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnMatchSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnGeocode() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnChangeTrip() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnUpdateAssociation() {
		// TODO Auto-generated method stub
		
	}
}
