package com.tripgis.dmt.ui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpringLayout;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.types.MatchRecord;
import com.tripgis.dmt.ui.listeners.EventButtonActionListener;
import com.tripgis.dmt.ui.listeners.EventEnabler;
import com.tripgis.dmt.ui.listeners.EventFormListener;
import com.tripgis.dmt.util.LogTranslator;

public class EventFormNearbyTab extends EventFormListener implements EventEnabler
{
    // Name to use in log lines
    public static final String TAG = EventFormNearbyTab.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );
	
	private JPanel container;
	private JList<MatchRecord> lstMatches;
	private MatchRecord selectedObject = null;

	JButton btnSetAssoc;
	JButton btnSearch;
	JButton btnRefresh;
	
	JTextField radius;
	JTextField keyword;

	private DefaultListModel<MatchRecord> matchList;
	private EventForm form;
	
	public EventFormNearbyTab()
	{
		Log.t( TAG, "EventFormNearbyTab()" );
	}
	
	public void initialize()
	{
		Log.t( TAG, "initialize()" );
		form = DmtController.getDisplay();
		initializeTab();
	}
	
	public void setNearbyLocations( ArrayList<MatchRecord> list )
	{
		int size = ( list == null ? 0 : list.size() );
		Log.t( TAG, "setNearbyLocations( " + size + " )" );
		
		form.enableListeners( false );
		//matchList.clear();
		matchList.removeAllElements();
		for( MatchRecord record : list )
			matchList.addElement( record );
//		lstMatches = new JList<>( matchList );
//		lstMatches.addListSelectionListener( this );
	       // Adding a list selection listener
		Log.d( TAG, "List Size: " + matchList.size() + ", Enabled: " + lstMatches.isEnabled() );
		form.enableListeners( true );
	}
	
	/**
	 * References:
	 *    http://www.codejava.net/java-se/swing/jlist-basic-tutorial-and-examples
	 */
	private void initializeTab()
	{
		Log.t( TAG, "initializeTab()" );
		
		container = new JPanel();		
		JPanel buttons = new JPanel();
		JPanel searchFields = new JPanel();

		matchList = new DefaultListModel<>();
		lstMatches = new JList<>( matchList ); 
		lstMatches.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		lstMatches.setLayoutOrientation( JList.HORIZONTAL_WRAP );
		lstMatches.setVisibleRowCount( -1 );
		//lstMatches.addListSelectionListener( this );
        lstMatches.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent e) {
            	Log.d( TAG, "ADJ: " + e.getValueIsAdjusting() );
                if (!e.getValueIsAdjusting()) {
                    MatchRecord selectedName = lstMatches.getSelectedValue();
                    selectedObject = selectedName;
                    Log.d( TAG, "OBJECT: " + selectedObject );
                    enableOnMatchSelected();
                }
            }
        });		

		JScrollPane listScroller = new JScrollPane( lstMatches );
		listScroller.setPreferredSize( new Dimension( 250, 200 ) );	

		btnSetAssoc = new JButton( EventForm.SET_ASSOC );
		btnSearch = new JButton( EventForm.SEARCH );
		btnRefresh = new JButton( EventForm.REFRESH );
		
		buttons.setLayout( new FlowLayout() );
		buttons.add( btnSetAssoc );
		buttons.add( btnSearch );
		buttons.add( btnRefresh );

		EventButtonActionListener listener = new EventButtonActionListener();
		btnSetAssoc.addActionListener( listener );
		btnSearch.addActionListener( listener );
		btnRefresh.addActionListener( listener );
		
		radius = new JTextField( 5 );
		radius.setToolTipText( "How far abroad to search (meters)" );
		radius.setText( "100" );
		keyword = new JTextField( 20 );

		JLabel[] labels     = new JLabel[2];
		labels[0] = new JLabel( "Radius" );
		labels[1] = new JLabel( "Keyword" );
		
		JComponent[] fields = new JComponent[2];
		fields[0] = radius;
		fields[1] = keyword;
		
		GroupLayout glayout = EventForm.prepareGroupLayout( searchFields, labels, fields );		
		searchFields.setLayout( glayout );
		glayout.setAutoCreateGaps( true );
		
		container.add( listScroller );
		container.add( buttons );
		container.add( searchFields );

		SpringLayout layout = new SpringLayout();
		container.setLayout( layout );
		layout.putConstraint( SpringLayout.HORIZONTAL_CENTER, listScroller, 0, SpringLayout.HORIZONTAL_CENTER, container );
		layout.putConstraint( SpringLayout.WEST, buttons, 5, SpringLayout.WEST, container );
		layout.putConstraint( SpringLayout.NORTH, listScroller, 5, SpringLayout.NORTH, container );
		layout.putConstraint( SpringLayout.NORTH, buttons, 5, SpringLayout.SOUTH, listScroller );
		layout.putConstraint( SpringLayout.NORTH, searchFields, 5, SpringLayout.SOUTH, buttons );
		layout.putConstraint( SpringLayout.EAST, container, 5, SpringLayout.EAST, buttons );
//		layout.putConstraint( SpringLayout.SOUTH, container, 5, SpringLayout.SOUTH, searchFields );
	}
	
	public JPanel getTab() { return container; }
	public String getRadius() { return radius.getText(); }
	public String getKeyword() { return keyword.getText(); }
	public MatchRecord getSelectedObject() { return selectedObject; }

	public JButton getSearchButton()             { return btnSearch;   }
	
	public void clear()
	{
//		matchList.removeAllElements();
		selectedObject = null;
//		matchList.clear();
//		lstMatches = new JList<>( matchList );
	}
	
	/*******************************************************
     * FORM FEATURE ENABLERS
     ******************************************************/
	
	@Override
	public void enableOnFormCreate() 
	{
		btnSetAssoc.setEnabled( false );
		btnSearch.setEnabled( false );
		btnRefresh.setEnabled( false ); // TODO - something with this
	}

	@Override
	public void enableOnChanged() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDbConnect() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDbDisconnect() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnOpenTrip() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveAs() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnBookmark() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSetEndTime() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveTrip() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void enableOnImportDb() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void enableOnExportDb(){
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnQgisOpen() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnQgisClose() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnAnalyzeOpen() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnAnalyzeClose() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnResetEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnDeleteEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSaveEvent() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnCloseTrip() 
	{
		btnSetAssoc.setEnabled( false );
		btnSearch.setEnabled( false );
	}

	@Override
	public void enableOnTripSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnEventSelected(boolean flag) 
	{
		btnSearch.setEnabled( flag );
	}

	@Override
	public void enableOnEventTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnEventSubTypeSelected() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnSetAssociation() 
	{
		clear();
		btnSetAssoc.setEnabled( false );
	}

	@Override
	public void enableOnSearch() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnRefresh() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnMatchSelected() 
	{
		btnSetAssoc.setEnabled( selectedObject != null );
	}

	@Override
	public void enableOnGeocode() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnChangeTrip() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void enableOnUpdateAssociation() {
		// TODO Auto-generated method stub
		
	}

	@Override
	protected void doPerformAction( String command ) 
	{
		Log.t( TAG, "doPerformAction( " + command + " )" );
/*		//lstMatches.setSelectedIndex( Integer.parseInt( command ) );
		Log.d( TAG, "MatchList1: " + matchList.getSize() );
		Log.d( TAG, "MatchList2: " + matchList );
		Log.d( TAG, "MatchList3: " + lstMatches.getSize() );
		Log.d( TAG, "MatchList4: " + lstMatches.getModel().getSize() );
		Log.d( TAG, "MatchList5: " + lstMatches );
		Log.d( TAG, "OBJECT: " + lstMatches.getSelectedValue() );
		//selectedObject = matchList.get( Integer.parseInt( command ) );
		selectedObject = lstMatches.getModel().getElementAt( Integer.parseInt( command ) );
		Log.d( TAG, "OBJECT: " + selectedObject );
		enableOnMatchSelected();

//		if( e.getValueIsAdjusting() == false )
		{
			if( lstMatches.getSelectedIndex() == -1 )
			{
				// perfectly obvious, the selected index -1 means selection!
				Log.d( TAG, "Selected " + getSelectedObject() + " - " + lstMatches.getSelectedIndex() );
			}
			else
			{
				Log.d( TAG, "Selected " + getSelectedObject() + " - " + lstMatches.getSelectedIndex() );
			}
			
			
		}
	*/	
		/*
		ListSelectionModel lsm;
		if( e.getSource() instanceof JList )
		{
			JList l = (JList) e.getSource();
			lsm = (ListSelectionModel) l.getModel();
		}
		else if( e.getSource() instanceof ListSelectionModel )
			lsm = (ListSelectionModel) e.getSource();
		else
		{
			Log.e( TAG, "Source: " + e.getSource() + " unknown." );
			return;
		}

		int firstIndex = e.getFirstIndex();
		int lastIndex = e.getLastIndex();
		boolean isAdjusting = e.getValueIsAdjusting();
		
		if( isAdjusting )
		{
			Log.d( TAG, "Yes, this is right!" );
			//TODO - btnSetAssoc.setEnabled( true );
		}

//		output.append("Event for indexes "
//		+ firstIndex + " - " + lastIndex
//		+ "; isAdjusting is " + isAdjusting
//		+ "; selected indexes:");

		if( lsm.isSelectionEmpty() )
			;//			output.append(" <none>");

		else
		{
			// Find out which indexes are selected.
			int minIndex = lsm.getMinSelectionIndex();
			int maxIndex = lsm.getMaxSelectionIndex();
			for( int i = minIndex; i <= maxIndex; i++ ) 
			{
				if( lsm.isSelectedIndex(i) )
				{
//					output.append(" " + i);
				}
			}
		}

//		output.append(newline);
		*/
	}
}
