package com.tripgis.dmt.ui.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.tripgis.dmt.DmtController;
import com.tripgis.dmt.types.EventRecordDisplayItem;
import com.tripgis.dmt.types.EventSubTypeDisplayItem;
import com.tripgis.dmt.types.EventTypeDisplayItem;
import com.tripgis.dmt.types.TripDbRecordDisplayItem;
import com.tripgis.dmt.ui.EventForm;
import com.tripgis.dmt.util.LogTranslator;

public abstract class EventFormListener implements 
	ActionListener, ItemListener, KeyListener, ListSelectionListener
{
	// Name to use in log lines
    public static final String TAG = EventFormListener.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );
    
    public static final String TRIP_SELECTED          = "onTripSelect";
    public static final String EVENT_SELECTED         = "onEventSelect";
    public static final String EVENT_TYPE_SELECTED    = "onEventTypeSelect";
    public static final String EVENT_SUBTYPE_SELECTED = "onEventSubTypeSelect";
    
	private static boolean active = false;
	public static void setActive( boolean flag ) { active = flag; }
	
	protected abstract void doPerformAction( String command );

    protected EventForm form;
    
    public void initialize()
    {
    	if( form == null )
    		form = DmtController.getDisplay();
    }

    @Override
	public void itemStateChanged( ItemEvent e ) 
    {
		initialize();
		if( active )
		{
			Log.t( TAG, "itemStateChanged( " + e.getItem() + " )" );
			Object item = e.getItem();
			if( item instanceof TripDbRecordDisplayItem )
				actOnSelection( TRIP_SELECTED, e.getStateChange() );

			else if( item instanceof EventRecordDisplayItem )
				actOnSelection( EVENT_SELECTED, e.getStateChange() );

			else if( item instanceof EventTypeDisplayItem )
				actOnSelection( EVENT_TYPE_SELECTED, e.getStateChange() );

			else if( item instanceof EventSubTypeDisplayItem )
				actOnSelection( EVENT_SUBTYPE_SELECTED, e.getStateChange() );
		}
    }
    
    private void actOnSelection( String command, int stateChange )
    {
		Log.t( TAG, "actOnSelection( " + command + ":" + 
				( stateChange == ItemEvent.SELECTED ? "SELECTED" : stateChange ) + " )" );
		if( stateChange == ItemEvent.SELECTED )
			doPerformAction( command );
    }

    /**
     * References:
     *    https://stackoverflow.com/questions/4722835/how-to-temporarily-disable-event-listeners-in-swing
     */
	@Override
	public void actionPerformed( ActionEvent e ) 
	{
		initialize();
		if( active )
		{
			Log.t( TAG, "actionPerformed( " + e + " )" );
			if( e.getSource() instanceof JButton )
			{
				JButton button = (JButton) e.getSource();
				doPerformAction( button.getText() );
			}
			else if( e.getSource() instanceof JMenuItem )
			{
				JMenuItem item = (JMenuItem) e.getSource();
				doPerformAction( item.getText() );
			}
			else
				Log.e( TAG, "Unrecognized source: " + e.getSource() );
		}
	}

	@Override
	public void keyPressed(KeyEvent arg0) 
	{
		initialize();
	}

	@Override
	public void keyReleased( KeyEvent e ) 
	{
		initialize();
		if( active )
		{
			Log.t( TAG, "keyReleased( " + e + " )" );
			String command = Integer.toString( e.getKeyCode() );
			doPerformAction( command );
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	
	/**
	 * References:
	 *    https://docs.oracle.com/javase/tutorial/uiswing/events/listselectionlistener.html
	 *    https://docs.oracle.com/javase/tutorial/uiswing/components/list.html
	 */
	@Override
	public void valueChanged( ListSelectionEvent e )
	{
		initialize();
		if( active )
		{
			Log.t( TAG, "valueChanged( " + e + " )" );
			Log.d( TAG, "First: " + e.getFirstIndex() + ", Last: " + e.getLastIndex() );
			Log.d( TAG, "Adjusting? " + e.getValueIsAdjusting() );
			if( !e.getValueIsAdjusting() )
				doPerformAction( Integer.toString( e.getFirstIndex() ) );
		}
	}
}
