package com.tripgis.dmt.util;

import java.io.Serializable;
import java.text.ParseException;
//import java.util.regex.Pattern;

public class LatLng implements Serializable 
{

	/**
	 * 
	 */
	private static final long serialVersionUID = 8754272445563088660L;

	// Name to use in log lines
    public static final String TAG = LatLng.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );
	
	public static final LatLng EMPTY = new LatLng( 0, 0 );
	
	public double latitude;
	public double longitude;
	
//	private static final Pattern stringMatch = Pattern.compile( "(-?\\d+\\.\\d+),(-?\\d+\\.\\d+)" );
	
	public LatLng( double lat, double lon )
	{
		latitude  = lat;
		longitude = lon;
	}
	
	/**
	 * References:
	 *    https://stackoverflow.com/questions/17969436/java-regex-capturing-groups
	 *    
	 * @param latlng
	 * @throws ParseException
	 */
	public LatLng( String latlng )
	{
		Log.t( TAG, "LatLng( " + latlng + " )" );
		if( latlng.isEmpty() )
		{
			latitude  = 0;
			longitude = 0;
			return;
		}
		
//		Log.d( TAG, "isEmpty: " + latlng.isEmpty() + ", null: " + ( latlng == null ) );
//		Log.d( TAG, "null: " + ( latlng == null ) );
//		Log.d( TAG, "Length: " + latlng.length() + ", equals: " + latlng.equals( "" ) );
		int start = latlng.indexOf( '(' );
		int mid = latlng.indexOf( ',' );
		int end = latlng.indexOf( ')' );
		if( end < 0 )
			end = latlng.length() - 1;
//		Log.d( TAG, "start: " + start + ", mid: " + mid + ", end: " + end );
		latitude  = Double.parseDouble( latlng.substring( start + 1, mid ) );
		longitude = Double.parseDouble( latlng.substring( mid + 1, end ) );
		Log.d( TAG, "Lat: " + latitude + ", Long: " + longitude );
	}
	
	public LatLng copy()
	{
		return new LatLng( latitude, longitude );
	}
	
	public boolean isValid() { return latitude != 0.0 && longitude != 0.0; }

    // Getter methods for Lat/Lon as string individually
    public String getLatitudeString() { return getDisplayString().split(",")[0]; }
    public String getLongitudeString() { return getDisplayString().split(",")[1]; }
	
    /**
     * Converts the location's LatLng object to the standard format.
     *
     * @return A string representation of Lat/Long
     */
	public String getDisplayString()
	{
        String lat = String.format( String.valueOf( latitude ), "%2.6f" );
        String lon = String.format( String.valueOf( longitude ), "%3.6f" );
        return lat + "," + lon;
	}
	
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "Lat=" + latitude + "," );
		sb.append( "Long=" + longitude );
		return sb.toString();
	}
}
