package com.tripgis.dmt.util;

import java.io.File;
import java.io.IOException;
import java.sql.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.TimeZone;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

public class LogTranslator 
{
	private Logger Log;
	private static FileAppender appender;

	private static boolean setup = false;
	
	/**
	 * References:
	 *    http://www.codejava.net/coding/how-to-configure-log4j-as-logging-mechanism-in-java?showall=&start=2#PropertiesFileConfiguration
	 *    https://logging.apache.org/log4j/1.2/apidocs/org/apache/log4j/PatternLayout.html
	 *    https://logging.apache.org/log4j/1.2/apidocs/org/apache/log4j/FileAppender.html
	 */
    public static void setupEnvironment() throws IOException
    {
    	if( !setup )
    	{
    		setup = true;
//        	System.setProperty( "current.date", "abc" + 
//        			DmtUtils.getTimestamp( LocationRecord.getCurrentTime() ) );
    		
    		// d - date, by default == %d{ISO8601}: yyyy-MM-dd HH:mm:ss,SSS
    		// r - time since start
    		// t - thread
    		// p - priority
    		// c - classname
    		// x - NDC with thread
    		// m - application supplied message
    		// n - line separator
    		//String pattern = "%d %-4r %-5p %c{1}: %m%n"
    		String pattern = "%d %-4r [%t] %-5p %c{1} %x: %m%n";
    		PatternLayout layout = new PatternLayout( pattern );
    		
    		DateFormat dateFormat = new SimpleDateFormat( "yyyy-MM-dd_hhmmss");
            Calendar calendar = Calendar.getInstance( TimeZone.getTimeZone( "UTC" ) );
            Date date = new Date( calendar.getTime().getTime() );
            String timestamp = dateFormat.format( date );
            
    		String fileName = System.getenv( "TRIPGIS_HOME" ) + File.separator + "logs" + 
    				File.separator + "log_" + timestamp + ".log";
    		appender = new FileAppender( layout, fileName ); //, false );
    		System.out.println( "Log File: " + appender.getFile() );
	   	}
    }
    	
    /**
     * References:
     *    https://www.tutorialspoint.com/log4j/log4j_logging_levels.htm
     * @param name
     */
	public LogTranslator( String name )
	{
		Log = Logger.getLogger( name );
		Log.setLevel( Level.ALL );

		try
		{
			setupEnvironment();
			Log.addAppender( appender );
		}
		catch( IOException e )
		{
			Log.error( "Failed to initialize logfile - " + e.getMessage() );
			e.printStackTrace();
		}
	}
	
	public int d( String tag, String text )
	{
		Log.debug( text );
		//System.out.println( "D/" + tag + ": " + text );
		return 0;
	}
	
	public int t( String tag, String text )
	{
		Log.trace( text );
		//System.out.println( "T/" + tag + ": " + text );
		return 0;
	}

	public int i( String tag, String text )
	{
		Log.info( text );
		//System.out.println( "I/" + tag + ": " + text );
		return 0;
	}
	
	public int w( String tag, String text )
	{
		Log.warn( text );
		//System.out.println( "W/" + tag + ": " + text );
		return 0;
	}

	public int e( String tag, String text )
	{
		Log.error( text );
		System.out.println( "E/" + tag + ": " + text );
		return 0;
	}
}
