package com.tripgis.shared.types;


import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;

import com.tripgis.dmt.DmtDataType;
import com.tripgis.dmt.DmtListType;
import com.tripgis.dmt.file.FileManager;
import com.tripgis.dmt.util.LogTranslator;

/**
 * LocationRecord is the data of import being collected by the Android App.
 *
 * Created by pk on 7/9/17.
 */

public class TripRecord implements Serializable, DmtDataType<TripRecord>, DmtListType<LocationRecord>
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 569376206613300388L;

    //TODO: Add additional map_items as they become known

    // Name to use in log lines
    public static final String TAG = TripRecord.class.getSimpleName();
    public static final LogTranslator Log = new LogTranslator( TAG );
    
    public static final TripRecord EMPTY = new TripRecord( "" );

    // Number of fields expected in a given record
    public static final int REQUIRED_FIELDS = 1;

    // Total number of fields in each record
    //   Used when writing records to pad each stored record with null fields
    public static final int TOTAL_NUM_FIELDS = REQUIRED_FIELDS + 0;
	
	private String tripName;                                // Title
	private String fileName;                                // File
    private ArrayList<LocationRecord> savedLocations;   // Stored items

    // Constructor
    public TripRecord( String name )
    {
        Log.d( TAG, "TripRecord( " + name + " )" );
        tripName = name;
        fileName = name;
        savedLocations = new ArrayList<>();
    }
    
    @SuppressWarnings("unchecked")
	public TripRecord copy()
    {
    	Log.t( TAG, "copy() = " + this );
    	TripRecord tr     = new TripRecord( tripName );
    	tr.fileName       = fileName;
    	tr.savedLocations = (ArrayList<LocationRecord>) savedLocations.clone();
    	return tr;
    }

    public String getStartTime()
    {
        LocationRecord record = getFirstRecord();
        if( record != null )
            return LocationRecord.getDateString( record.getDate() );

        return "";
    }

    public String getEndTime()
    {
        LocationRecord record = getLastRecord();
        if( record != null )
        {
            String text = LocationRecord.getDateString( record.getEndTime() );
            if( text.equals( "" ) )
                return LocationRecord.getDateString( record.getDate() );
        }

        return "";
    }

    // Setter methods
    public void setTripName( String n )     { fileName = n; }
    public void setFileName( String n )     { tripName = n; }
//    public void setSavedLocations( ArrayList<LocationRecord> list ) { savedLocations = list; }

    public String getTripName() { return fileName; }
    public String getFileName() { return tripName; }
//    public ArrayList<LocationRecord> getSavedLocations() { return savedLocations; }

    public LocationRecord getFirstRecord() { return savedLocations.get( 0 ); }
    public LocationRecord getLastRecord() { return savedLocations.get( savedLocations.size() - 1 ); }
    public int getItemCount() { return savedLocations.size(); }

	public static TripRecord buildFromCsvArray( String[] data ) throws IOException
	{
		Log.t( TAG, "buildFromCsvArray( " + data.length + " )" );

        // Verify the string array passed in has the correct number of fields
        if( data.length != REQUIRED_FIELDS )
        {
    		throw new IOException( "Invalid number of parameters (" + data.length + " vs. " + REQUIRED_FIELDS + ")" );
        }

        Log.d( TAG, "Creating record from " + "'" + data[0] + "'..." );
        TripRecord tr = new TripRecord( data[0] );
//TODO -        tr.setFileName( data[1] );
        return tr;
	}

	@Override
	public String generateCsvString() 
	{
        StringBuffer sb = new StringBuffer();
        sb.append( tripName );
        sb.append( FileManager.CSV_SEPARATOR );
        sb.append( fileName );
        return sb.toString();
	}

	@Override
	public boolean matches( TripRecord t ) { return t.tripName.equals( tripName ); }

    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append( "TripName=" + tripName + "," );
        sb.append( "FileName=" + fileName + "," );
        sb.append( "NumLocations=" + savedLocations.size() );
        return sb.toString();
    }

	@Override
	public void setList( ArrayList<LocationRecord> list ) 
	{
		savedLocations = list; 
		Log.d( TAG, "setList( " + list.size() + " ) = " + savedLocations.size() );
		Log.d( TAG, "OBJ: " + this );
	}

	@Override
	public ArrayList<LocationRecord> getList() 
	{ 
		Log.d( TAG, "getList() = " + savedLocations.size() );
		return savedLocations; 
	}
}
